from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import thread_extend

expected_verilog = """
module test;

  reg CLK;
  reg RST;
  wire [8-1:0] LED;

  blinkled
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .LED(LED)
  );


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #10000;
    $finish;
  end


endmodule



module blinkled
(
  input CLK,
  input RST,
  output reg [8-1:0] LED
);

  reg [32-1:0] fsm;
  localparam fsm_init = 0;
  reg signed [32-1:0] _th_blink_times_0;
  reg signed [32-1:0] _th_blink_i_1;
  localparam fsm_1 = 1;
  localparam fsm_2 = 2;
  localparam fsm_3 = 3;
  localparam fsm_4 = 4;
  localparam fsm_5 = 5;
  localparam fsm_6 = 6;
  localparam fsm_7 = 7;
  localparam fsm_8 = 8;
  localparam fsm_9 = 9;

  always @(posedge CLK) begin
    if(RST) begin
      fsm <= fsm_init;
      LED <= 0;
      _th_blink_times_0 <= 0;
      _th_blink_i_1 <= 0;
    end else begin
      case(fsm)
        fsm_init: begin
          LED <= 100;
          fsm <= fsm_1;
        end
        fsm_1: begin
          LED <= 200;
          fsm <= fsm_2;
        end
        fsm_2: begin
          _th_blink_times_0 <= 10;
          fsm <= fsm_3;
        end
        fsm_3: begin
          LED <= 0;
          fsm <= fsm_4;
        end
        fsm_4: begin
          _th_blink_i_1 <= 0;
          fsm <= fsm_5;
        end
        fsm_5: begin
          if(_th_blink_i_1 < _th_blink_times_0) begin
            fsm <= fsm_6;
          end else begin
            fsm <= fsm_9;
          end
        end
        fsm_6: begin
          LED <= LED + 1;
          fsm <= fsm_7;
        end
        fsm_7: begin
          $display("led =  %d", LED);
          fsm <= fsm_8;
        end
        fsm_8: begin
          _th_blink_i_1 <= _th_blink_i_1 + 1;
          fsm <= fsm_5;
        end
      endcase
    end
  end


endmodule
"""


def test():
    veriloggen.reset()
    test_module = thread_extend.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import collections

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *


def mkLed():
    m = Module('blinkled')
    width = m.Parameter('WIDTH', 8)
    clk = m.Input('CLK')
    rst = m.Input('RST')
    led = m.OutputReg('LED', width)

    dummy_out0 = m.Output('dummy_out0', width)
    dummy_out1 = m.Output('dummy_out1', width)
    dummy_out2 = m.Output('dummy_out2', width)
    dummy_in0 = m.Input('dummy_in0', width)
    dummy_in1 = m.Input('dummy_in1', width)
    dummy_in2 = m.Input('dummy_in2', width)

    count = m.Reg('count', 32)

    m.Always(Posedge(clk))(
        If(rst)(
            count(0)
        ).Else(
            If(count == 1023)(
                count(0)
            ).Else(
                count(count + 1)
            )
        ))

    m.Always(Posedge(clk))(
        If(rst)(
            led(0)
        ).Else(
            If(count == 1023)(
                led(led + 1)
            )
        ))

    return m


def mkTop():
    m = Module('top')
    width = m.Parameter('WIDTH', 8)
    clk = m.Input('CLK')
    rst = m.Input('RST')
    led = m.Output('LED', width)

    sub = Submodule(m, mkLed(), 'inst_blinkled',
                    arg_params=(width,),
                    arg_ports=(clk, rst, led),
                    as_io=('dummy_out0', 'dummy_in0'), as_wire=('dummy_out1', 'dummy_in1'))

    return m

if __name__ == '__main__':
    top = mkTop()
    verilog = top.to_verilog('tmp.v')
    print(verilog)



from __future__ import absolute_import
from __future__ import print_function

from collections import OrderedDict
from tempfile import NamedTemporaryFile

import ipgen.ipgen

import veriloggen.core.vtypes as vtypes
import veriloggen.core.module as module
from . import axi


def to_ipcore(m, ipname=None, topname=None,
              clkname='CLK', rstname='RST', iftype='axi', use_acp=False,
              simaddrwidth=20, simcode=None, simmemimg=None,
              silent=False):

    if topname is None:
        topname = '_'.join(['top', m.name])

    if ipname is None:
        ipname = '_'.join([m.name, 'ip'])

    top = generate_top(m, topname, clkname, rstname)
    verilog = top.to_verilog()

    run_ipgen(topname, ipname, verilog,
              iftype, use_acp,
              simaddrwidth, simcode, simmemimg,
              silent)


def run_ipgen(topname, ipname, verilog,
              iftype='axi', use_acp=False,
              simaddrwidth=20, simcode=None, simmemimg=None,
              silent=False):

    # memory image
    if simmemimg is None:
        size = 2 ** simaddrwidth
        wordsize = 4
        file_memimg = NamedTemporaryFile('w+')
        axi.AxiMemoryModel._make_img(file_memimg.name, size, wordsize)
        file_memimg.read()
        file_memimg_name = file_memimg.name
    else:
        file_memimg = None
        file_memimg_name = simmemimg

    # source code
    file_source = NamedTemporaryFile('w+')
    file_source.write(verilog)
    file_source.read()

    # usertest
    file_simcode = None
    if simcode is not None:
        file_simcode = NamedTemporaryFile('w+')
        file_simcode.write(simcode)
        file_simcode.read()

    # configuration
    topmodule = topname
    userlogic_filelist = [file_source.name]
    include = []
    define = []
    usertest = file_simcode.name if file_simcode is not None else None
    memimg = file_memimg_name
    skip_not_found = False
    ignore_protocol_error = False

    configs = {
        'signal_width': 32,
        'ext_addrwidth': 32,
        'ext_datawidth': 256,
        'single_clock': True,
        'if_type': iftype,
        'use_acp': use_acp,
        'output': 'tmp.v',
        'sim_addrwidth': simaddrwidth,
        'hperiod_ulogic': 5,
        'hperiod_bus': 5,
    }

    builder = ipgen.ipgen.SystemBuilder()
    builder.build(configs,
                  topmodule,
                  ipname,
                  userlogic_filelist,

                  include=include,
                  define=define,
                  usertest=usertest,
                  memimg=memimg,

                  ignore_protocol_error=ignore_protocol_error,
                  skip_not_found=skip_not_found,
                  silent=silent)

    if file_memimg is not None:
        file_memimg.close()

    file_source.close()

    if file_simcode:
        file_simcode.close()


def generate_top(m, name, clkname='CLK', rstname='RST'):

    if not isinstance(m, module.Module):
        raise TypeError("'m' must be Module, not %s" % str(type(m)))

    masterbus = m.masterbus if hasattr(m, 'masterbus') else ()
    slavebus = m.slavebus if hasattr(m, 'slavebus') else ()

    include_names = [master.name for master in masterbus]
    include_names += [slave.name for slave in slavebus]

    top = module.Module(name)

    params = top.copy_params(m)
    ports = top.copy_ports(m, exclude=tuple(include_names))
    bus_ports = top.copy_sim_ports(
        m, include=tuple(include_names), use_wire=True)

    body = top.Instance(m, 'inst_' + m.name,
                        params=top.connect_params(m), ports=top.connect_ports(m))

    clk = ports[clkname]
    rst = ports[rstname]

    _id_count = 0

    for master in masterbus:
        port_list = OrderedDict([(k, v) for k, v in top.get_vars().items()
                                 if k.startswith(master.name)])
        name = master.name
        _id = _id_count
        _id_count += 1
        addr_width = port_list[master.name + '_awaddr'].width
        data_width = port_list[master.name + '_wdata'].width

        mod = ipgen_master_lite_memory() if master.lite else ipgen_master_memory()
        inst_params = (('NAME', name), ('ID', _id),
                       ('ADDR_WIDTH', addr_width), ('DATA_WIDTH', data_width))
        inst_ports = [('CLK', clk), ('RST', rst)]
        inst_ports += top.connect_ports(mod, prefix=name + '_')
        top.Instance(mod, 'inst_' + name, params=inst_params, ports=inst_ports)

    for slave in slavebus:
        port_list = OrderedDict([(k, v) for k, v in top.get_vars().items()
                                 if k.startswith(slave.name)])
        name = slave.name
        _id = _id_count
        _id_count += 1
        addr_width = port_list[slave.name + '_awaddr'].width
        data_width = port_list[slave.name + '_wdata'].width

        mod = ipgen_slave_lite_memory() if slave.lite else ipgen_slave_memory()
        inst_params = (('NAME', name), ('ID', _id),
                       ('ADDR_WIDTH', addr_width), ('DATA_WIDTH', data_width))
        inst_ports = [('CLK', clk), ('RST', rst)]
        inst_ports += top.connect_ports(mod, prefix=name + '_')
        top.Instance(mod, 'inst_' + name, params=inst_params, ports=inst_ports)

    return top


def mk_ipgen_master_memory():
    m = module.Module('ipgen_master_memory')

    name = m.Parameter('NAME', 'undefined')
    _id = m.Parameter('ID', 0)
    addr_width = m.Parameter('ADDR_WIDTH', 32)
    data_width = m.Parameter('DATA_WIDTH', 32)

    clk = m.Input('CLK')
    rst = m.Input('RST')

    awvalid = m.Input('awvalid')
    awaddr = m.Input('awaddr', addr_width)
    awlen = m.Input('awlen', 8)
    awready = m.Output('awready')

    wdata = m.Input('wdata', data_width)
    wstrb = m.Input('wstrb', data_width / 8)
    wlast = m.Input('wlast')
    wvalid = m.Input('wvalid')
    wready = m.Output('wready')

    arvalid = m.Input('arvalid')
    araddr = m.Input('araddr', addr_width)
    arlen = m.Input('arlen', 8)
    arready = m.Output('arready')

    rdata = m.Output('rdata', data_width)
    rlast = m.Output('rlast')
    rvalid = m.Output('rvalid')
    rready = m.Input('rready')

    return m


def mk_ipgen_slave_memory():
    m = module.Module('ipgen_slave_memory')

    name = m.Parameter('NAME', 'undefined')
    _id = m.Parameter('ID', 0)
    addr_width = m.Parameter('ADDR_WIDTH', 32)
    data_width = m.Parameter('DATA_WIDTH', 32)

    clk = m.Input('CLK')
    rst = m.Input('RST')

    awvalid = m.Output('awvalid')
    awaddr = m.Output('awaddr', addr_width)
    awlen = m.Output('awlen', 8)
    awready = m.Input('awready')

    wdata = m.Output('wdata', data_width)
    wstrb = m.Output('wstrb', data_width / 8)
    wlast = m.Output('wlast')
    wvalid = m.Output('wvalid')
    wready = m.Input('wready')

    arvalid = m.Output('arvalid')
    araddr = m.Output('araddr', addr_width)
    arlen = m.Output('arlen', 8)
    arready = m.Input('arready')

    rdata = m.Input('rdata', data_width)
    rlast = m.Input('rlast')
    rvalid = m.Input('rvalid')
    rready = m.Output('rready')

    return m


def mk_ipgen_master_lite_memory():
    m = module.Module('ipgen_master_lite_memory')

    name = m.Parameter('NAME', 'undefined')
    _id = m.Parameter('ID', 0)
    addr_width = m.Parameter('ADDR_WIDTH', 32)
    data_width = m.Parameter('DATA_WIDTH', 32)

    clk = m.Input('CLK')
    rst = m.Input('RST')

    awvalid = m.Input('awvalid')
    awaddr = m.Input('awaddr', addr_width)
    awready = m.Output('awready')

    wdata = m.Input('wdata', data_width)
    wstrb = m.Input('wstrb', data_width / 8)
    wvalid = m.Input('wvalid')
    wready = m.Output('wready')

    arvalid = m.Input('arvalid')
    araddr = m.Input('araddr', addr_width)
    arready = m.Output('arready')

    rdata = m.Output('rdata', data_width)
    rvalid = m.Output('rvalid')
    rready = m.Input('rready')

    return m


def mk_ipgen_slave_lite_memory():
    m = module.Module('ipgen_slave_lite_memory')

    name = m.Parameter('NAME', 'undefined')
    _id = m.Parameter('ID', 0)
    addr_width = m.Parameter('ADDR_WIDTH', 32)
    data_width = m.Parameter('DATA_WIDTH', 32)

    clk = m.Input('CLK')
    rst = m.Input('RST')

    awvalid = m.Output('awvalid')
    awaddr = m.Output('awaddr', addr_width)
    awready = m.Input('awready')

    wdata = m.Output('wdata', data_width)
    wstrb = m.Output('wstrb', data_width / 8)
    wvalid = m.Output('wvalid')
    wready = m.Input('wready')

    arvalid = m.Output('arvalid')
    araddr = m.Output('araddr', addr_width)
    arready = m.Input('arready')

    rdata = m.Input('rdata', data_width)
    rvalid = m.Input('rvalid')
    rready = m.Output('rready')

    return m


_master_memory = None
_slave_memory = None
_master_lite_memory = None
_slave_lite_memory = None


def ipgen_master_memory():
    global _master_memory
    if _master_memory is None:
        _master_memory = mk_ipgen_master_memory()
    return _master_memory


def ipgen_slave_memory():
    global _slave_memory
    if _slave_memory is None:
        _slave_memory = mk_ipgen_slave_memory()
    return _slave_memory


def ipgen_master_lite_memory():
    global _master_lite_memory
    if _master_lite_memory is None:
        _master_lite_memory = mk_ipgen_master_lite_memory()
    return _master_lite_memory


def ipgen_slave_lite_memory():
    global _slave_lite_memory
    if _slave_lite_memory is None:
        _slave_lite_memory = mk_ipgen_slave_lite_memory()
    return _slave_lite_memory



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import thread_multibank_ram

expected_verilog = """
module test;

  reg CLK;
  reg RST;

  blinkled
  uut
  (
    .CLK(CLK),
    .RST(RST)
  );


  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #20000;
    $finish;
  end


endmodule



module blinkled
(
  input CLK,
  input RST
);

  reg [10-1:0] myram_0_0_addr;
  wire [32-1:0] myram_0_0_rdata;
  reg [32-1:0] myram_0_0_wdata;
  reg myram_0_0_wenable;

  myram_0
  inst_myram_0
  (
    .CLK(CLK),
    .myram_0_0_addr(myram_0_0_addr),
    .myram_0_0_rdata(myram_0_0_rdata),
    .myram_0_0_wdata(myram_0_0_wdata),
    .myram_0_0_wenable(myram_0_0_wenable)
  );

  reg [10-1:0] myram_1_0_addr;
  wire [32-1:0] myram_1_0_rdata;
  reg [32-1:0] myram_1_0_wdata;
  reg myram_1_0_wenable;

  myram_1
  inst_myram_1
  (
    .CLK(CLK),
    .myram_1_0_addr(myram_1_0_addr),
    .myram_1_0_rdata(myram_1_0_rdata),
    .myram_1_0_wdata(myram_1_0_wdata),
    .myram_1_0_wenable(myram_1_0_wenable)
  );

  reg [10-1:0] myram_2_0_addr;
  wire [32-1:0] myram_2_0_rdata;
  reg [32-1:0] myram_2_0_wdata;
  reg myram_2_0_wenable;

  myram_2
  inst_myram_2
  (
    .CLK(CLK),
    .myram_2_0_addr(myram_2_0_addr),
    .myram_2_0_rdata(myram_2_0_rdata),
    .myram_2_0_wdata(myram_2_0_wdata),
    .myram_2_0_wenable(myram_2_0_wenable)
  );

  reg [10-1:0] myram_3_0_addr;
  wire [32-1:0] myram_3_0_rdata;
  reg [32-1:0] myram_3_0_wdata;
  reg myram_3_0_wenable;

  myram_3
  inst_myram_3
  (
    .CLK(CLK),
    .myram_3_0_addr(myram_3_0_addr),
    .myram_3_0_rdata(myram_3_0_rdata),
    .myram_3_0_wdata(myram_3_0_wdata),
    .myram_3_0_wenable(myram_3_0_wenable)
  );

  reg [32-1:0] th_blink;
  localparam th_blink_init = 0;
  reg signed [32-1:0] _th_blink_times_0;
  reg signed [32-1:0] _th_blink_i_1;
  reg signed [32-1:0] _th_blink_wdata_2;
  wire [2-1:0] _tmp_0;
  assign _tmp_0 = _th_blink_i_1;
  reg _myram_0_cond_0_1;
  reg _myram_1_cond_0_1;
  reg _myram_2_cond_0_1;
  reg _myram_3_cond_0_1;
  reg signed [32-1:0] _th_blink_sum_3;
  wire [2-1:0] _tmp_1;
  assign _tmp_1 = _th_blink_i_1;
  reg _tmp_2;
  reg _myram_0_cond_1_1;
  reg _myram_0_cond_2_1;
  reg _myram_0_cond_2_2;
  reg _tmp_3;
  reg _myram_1_cond_1_1;
  reg _myram_1_cond_2_1;
  reg _myram_1_cond_2_2;
  reg _tmp_4;
  reg _myram_2_cond_1_1;
  reg _myram_2_cond_2_1;
  reg _myram_2_cond_2_2;
  reg _tmp_5;
  reg _myram_3_cond_1_1;
  reg _myram_3_cond_2_1;
  reg _myram_3_cond_2_2;
  reg signed [32-1:0] _tmp_6;
  reg signed [32-1:0] _th_blink_rdata_4;

  always @(posedge CLK) begin
    if(RST) begin
      myram_0_0_addr <= 0;
      myram_0_0_wdata <= 0;
      myram_0_0_wenable <= 0;
      _myram_0_cond_0_1 <= 0;
      _myram_0_cond_1_1 <= 0;
      _tmp_2 <= 0;
      _myram_0_cond_2_1 <= 0;
      _myram_0_cond_2_2 <= 0;
    end else begin
      if(_myram_0_cond_2_2) begin
        _tmp_2 <= 0;
      end 
      if(_myram_0_cond_0_1) begin
        myram_0_0_wenable <= 0;
      end 
      if(_myram_0_cond_1_1) begin
        _tmp_2 <= 1;
      end 
      _myram_0_cond_2_2 <= _myram_0_cond_2_1;
      if((th_blink == 4) && (_tmp_0 == 0)) begin
        myram_0_0_addr <= _th_blink_i_1 >> 2;
        myram_0_0_wdata <= _th_blink_wdata_2;
        myram_0_0_wenable <= 1;
      end 
      _myram_0_cond_0_1 <= (th_blink == 4) && (_tmp_0 == 0);
      if(th_blink == 10) begin
        myram_0_0_addr <= _th_blink_i_1 >> 2;
      end 
      _myram_0_cond_1_1 <= th_blink == 10;
      _myram_0_cond_2_1 <= th_blink == 10;
    end
  end


  always @(posedge CLK) begin
    if(RST) begin
      myram_1_0_addr <= 0;
      myram_1_0_wdata <= 0;
      myram_1_0_wenable <= 0;
      _myram_1_cond_0_1 <= 0;
      _myram_1_cond_1_1 <= 0;
      _tmp_3 <= 0;
      _myram_1_cond_2_1 <= 0;
      _myram_1_cond_2_2 <= 0;
    end else begin
      if(_myram_1_cond_2_2) begin
        _tmp_3 <= 0;
      end 
      if(_myram_1_cond_0_1) begin
        myram_1_0_wenable <= 0;
      end 
      if(_myram_1_cond_1_1) begin
        _tmp_3 <= 1;
      end 
      _myram_1_cond_2_2 <= _myram_1_cond_2_1;
      if((th_blink == 4) && (_tmp_0 == 1)) begin
        myram_1_0_addr <= _th_blink_i_1 >> 2;
        myram_1_0_wdata <= _th_blink_wdata_2;
        myram_1_0_wenable <= 1;
      end 
      _myram_1_cond_0_1 <= (th_blink == 4) && (_tmp_0 == 1);
      if(th_blink == 10) begin
        myram_1_0_addr <= _th_blink_i_1 >> 2;
      end 
      _myram_1_cond_1_1 <= th_blink == 10;
      _myram_1_cond_2_1 <= th_blink == 10;
    end
  end


  always @(posedge CLK) begin
    if(RST) begin
      myram_2_0_addr <= 0;
      myram_2_0_wdata <= 0;
      myram_2_0_wenable <= 0;
      _myram_2_cond_0_1 <= 0;
      _myram_2_cond_1_1 <= 0;
      _tmp_4 <= 0;
      _myram_2_cond_2_1 <= 0;
      _myram_2_cond_2_2 <= 0;
    end else begin
      if(_myram_2_cond_2_2) begin
        _tmp_4 <= 0;
      end 
      if(_myram_2_cond_0_1) begin
        myram_2_0_wenable <= 0;
      end 
      if(_myram_2_cond_1_1) begin
        _tmp_4 <= 1;
      end 
      _myram_2_cond_2_2 <= _myram_2_cond_2_1;
      if((th_blink == 4) && (_tmp_0 == 2)) begin
        myram_2_0_addr <= _th_blink_i_1 >> 2;
        myram_2_0_wdata <= _th_blink_wdata_2;
        myram_2_0_wenable <= 1;
      end 
      _myram_2_cond_0_1 <= (th_blink == 4) && (_tmp_0 == 2);
      if(th_blink == 10) begin
        myram_2_0_addr <= _th_blink_i_1 >> 2;
      end 
      _myram_2_cond_1_1 <= th_blink == 10;
      _myram_2_cond_2_1 <= th_blink == 10;
    end
  end


  always @(posedge CLK) begin
    if(RST) begin
      myram_3_0_addr <= 0;
      myram_3_0_wdata <= 0;
      myram_3_0_wenable <= 0;
      _myram_3_cond_0_1 <= 0;
      _myram_3_cond_1_1 <= 0;
      _tmp_5 <= 0;
      _myram_3_cond_2_1 <= 0;
      _myram_3_cond_2_2 <= 0;
    end else begin
      if(_myram_3_cond_2_2) begin
        _tmp_5 <= 0;
      end 
      if(_myram_3_cond_0_1) begin
        myram_3_0_wenable <= 0;
      end 
      if(_myram_3_cond_1_1) begin
        _tmp_5 <= 1;
      end 
      _myram_3_cond_2_2 <= _myram_3_cond_2_1;
      if((th_blink == 4) && (_tmp_0 == 3)) begin
        myram_3_0_addr <= _th_blink_i_1 >> 2;
        myram_3_0_wdata <= _th_blink_wdata_2;
        myram_3_0_wenable <= 1;
      end 
      _myram_3_cond_0_1 <= (th_blink == 4) && (_tmp_0 == 3);
      if(th_blink == 10) begin
        myram_3_0_addr <= _th_blink_i_1 >> 2;
      end 
      _myram_3_cond_1_1 <= th_blink == 10;
      _myram_3_cond_2_1 <= th_blink == 10;
    end
  end

  localparam th_blink_1 = 1;
  localparam th_blink_2 = 2;
  localparam th_blink_3 = 3;
  localparam th_blink_4 = 4;
  localparam th_blink_5 = 5;
  localparam th_blink_6 = 6;
  localparam th_blink_7 = 7;
  localparam th_blink_8 = 8;
  localparam th_blink_9 = 9;
  localparam th_blink_10 = 10;
  localparam th_blink_11 = 11;
  localparam th_blink_12 = 12;
  localparam th_blink_13 = 13;
  localparam th_blink_14 = 14;
  localparam th_blink_15 = 15;
  localparam th_blink_16 = 16;

  always @(posedge CLK) begin
    if(RST) begin
      th_blink <= th_blink_init;
      _th_blink_times_0 <= 0;
      _th_blink_i_1 <= 0;
      _th_blink_wdata_2 <= 0;
      _th_blink_sum_3 <= 0;
      _tmp_6 <= 0;
      _th_blink_rdata_4 <= 0;
    end else begin
      case(th_blink)
        th_blink_init: begin
          _th_blink_times_0 <= 10;
          th_blink <= th_blink_1;
        end
        th_blink_1: begin
          _th_blink_i_1 <= 0;
          th_blink <= th_blink_2;
        end
        th_blink_2: begin
          if(_th_blink_i_1 < (_th_blink_times_0 << 2)) begin
            th_blink <= th_blink_3;
          end else begin
            th_blink <= th_blink_7;
          end
        end
        th_blink_3: begin
          _th_blink_wdata_2 <= _th_blink_i_1;
          th_blink <= th_blink_4;
        end
        th_blink_4: begin
          th_blink <= th_blink_5;
        end
        th_blink_5: begin
          $display("wdata = %d", _th_blink_wdata_2);
          th_blink <= th_blink_6;
        end
        th_blink_6: begin
          _th_blink_i_1 <= _th_blink_i_1 + 1;
          th_blink <= th_blink_2;
        end
        th_blink_7: begin
          _th_blink_sum_3 <= 0;
          th_blink <= th_blink_8;
        end
        th_blink_8: begin
          _th_blink_i_1 <= 0;
          th_blink <= th_blink_9;
        end
        th_blink_9: begin
          if(_th_blink_i_1 < (_th_blink_times_0 << 2)) begin
            th_blink <= th_blink_10;
          end else begin
            th_blink <= th_blink_15;
          end
        end
        th_blink_10: begin
          if(_tmp_2 && (_tmp_1 == 0)) begin
            _tmp_6 <= myram_0_0_rdata;
          end 
          if(_tmp_3 && (_tmp_1 == 1)) begin
            _tmp_6 <= myram_1_0_rdata;
          end 
          if(_tmp_4 && (_tmp_1 == 2)) begin
            _tmp_6 <= myram_2_0_rdata;
          end 
          if(_tmp_5 && (_tmp_1 == 3)) begin
            _tmp_6 <= myram_3_0_rdata;
          end 
          if(_tmp_2) begin
            th_blink <= th_blink_11;
          end 
        end
        th_blink_11: begin
          _th_blink_rdata_4 <= _tmp_6;
          th_blink <= th_blink_12;
        end
        th_blink_12: begin
          _th_blink_sum_3 <= _th_blink_sum_3 + _th_blink_rdata_4;
          th_blink <= th_blink_13;
        end
        th_blink_13: begin
          $display("rdata = %d", _th_blink_rdata_4);
          th_blink <= th_blink_14;
        end
        th_blink_14: begin
          _th_blink_i_1 <= _th_blink_i_1 + 1;
          th_blink <= th_blink_9;
        end
        th_blink_15: begin
          $display("sum = %d", _th_blink_sum_3);
          th_blink <= th_blink_16;
        end
      endcase
    end
  end


endmodule



module myram_0
(
  input CLK,
  input [10-1:0] myram_0_0_addr,
  output [32-1:0] myram_0_0_rdata,
  input [32-1:0] myram_0_0_wdata,
  input myram_0_0_wenable
);

  reg [10-1:0] myram_0_0_daddr;
  reg [32-1:0] mem [0:1024-1];

  always @(posedge CLK) begin
    if(myram_0_0_wenable) begin
      mem[myram_0_0_addr] <= myram_0_0_wdata;
    end 
    myram_0_0_daddr <= myram_0_0_addr;
  end

  assign myram_0_0_rdata = mem[myram_0_0_daddr];

endmodule



module myram_1
(
  input CLK,
  input [10-1:0] myram_1_0_addr,
  output [32-1:0] myram_1_0_rdata,
  input [32-1:0] myram_1_0_wdata,
  input myram_1_0_wenable
);

  reg [10-1:0] myram_1_0_daddr;
  reg [32-1:0] mem [0:1024-1];

  always @(posedge CLK) begin
    if(myram_1_0_wenable) begin
      mem[myram_1_0_addr] <= myram_1_0_wdata;
    end 
    myram_1_0_daddr <= myram_1_0_addr;
  end

  assign myram_1_0_rdata = mem[myram_1_0_daddr];

endmodule



module myram_2
(
  input CLK,
  input [10-1:0] myram_2_0_addr,
  output [32-1:0] myram_2_0_rdata,
  input [32-1:0] myram_2_0_wdata,
  input myram_2_0_wenable
);

  reg [10-1:0] myram_2_0_daddr;
  reg [32-1:0] mem [0:1024-1];

  always @(posedge CLK) begin
    if(myram_2_0_wenable) begin
      mem[myram_2_0_addr] <= myram_2_0_wdata;
    end 
    myram_2_0_daddr <= myram_2_0_addr;
  end

  assign myram_2_0_rdata = mem[myram_2_0_daddr];

endmodule



module myram_3
(
  input CLK,
  input [10-1:0] myram_3_0_addr,
  output [32-1:0] myram_3_0_rdata,
  input [32-1:0] myram_3_0_wdata,
  input myram_3_0_wenable
);

  reg [10-1:0] myram_3_0_daddr;
  reg [32-1:0] mem [0:1024-1];

  always @(posedge CLK) begin
    if(myram_3_0_wenable) begin
      mem[myram_3_0_addr] <= myram_3_0_wdata;
    end 
    myram_3_0_daddr <= myram_3_0_addr;
  end

  assign myram_3_0_rdata = mem[myram_3_0_daddr];

endmodule
"""


def test():
    veriloggen.reset()
    test_module = thread_multibank_ram.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function

import math
import veriloggen.core.vtypes as vtypes
from veriloggen.core.module import Module
from veriloggen.core.submodule import Submodule
from veriloggen.fsm.fsm import FSM

from .ttypes import _MutexFunction


class UartTx(Submodule, _MutexFunction):
    __intrinsics__ = ('send',) + _MutexFunction.__intrinsics__

    def __init__(self, m, name, prefix, clk, rst, txd=None,
                 arg_params=None, arg_ports=None,
                 as_io=None, as_wire=None,
                 baudrate=19200, clockfreq=100 * 1000 * 1000):

        if arg_ports is None:
            arg_ports = []

        arg_ports.insert(0, ('CLK', clk))
        arg_ports.insert(1, ('RST', rst))

        if txd is not None:
            arg_ports.insert(2, ('txd', txd))

        moddef = mkUartTx(baudrate, clockfreq)

        Submodule.__init__(self, m, moddef, name, prefix,
                           arg_params=arg_params, arg_ports=arg_ports,
                           as_io=as_io, as_wire=as_wire)

        self.tx_din = self['din']
        self.tx_enable = self['enable']
        self.tx_enable.initval = 0
        self.tx_ready = self['ready']

        self.mutex = None

    def send(self, fsm, value):
        fsm(
            self.tx_din(value),
            self.tx_enable(1)
        )
        fsm.goto_next()
        fsm(
            self.tx_enable(0)
        )
        fsm.goto_next()
        fsm.If(self.tx_ready).goto_next()


class UartRx(Submodule, _MutexFunction):
    __intrinsics__ = ('recv',) + _MutexFunction.__intrinsics__

    def __init__(self, m, name, prefix, clk, rst, rxd=None,
                 arg_params=None, arg_ports=None,
                 as_io=None, as_wire=None,
                 baudrate=19200, clockfreq=100 * 1000 * 1000):

        if arg_ports is None:
            arg_ports = []

        arg_ports.insert(0, ('CLK', clk))
        arg_ports.insert(1, ('RST', rst))

        if rxd is not None:
            arg_ports.insert(2, ('rxd', rxd))

        moddef = mkUartRx(baudrate, clockfreq)
        Submodule.__init__(self, m, moddef, name, prefix,
                           arg_params=arg_params, arg_ports=arg_ports,
                           as_io=as_io, as_wire=as_wire)

        self.rx_dout = self['dout']
        self.rx_valid = self['valid']

        self.mutex = None

    def recv(self, fsm):
        ret = fsm.m.TmpReg(self.rx_dout.width)
        fsm.If(self.rx_valid)(
            ret(self.rx_dout)
        )
        fsm.Then().goto_next()
        return ret


def mkUartTx(baudrate=19200, clockfreq=100 * 1000 * 1000):
    m = Module("UartTx")
    waitnum = int(clockfreq / baudrate)

    clk = m.Input('CLK')
    rst = m.Input('RST')

    din = m.Input('din', 8)
    enable = m.Input('enable')
    ready = m.OutputReg('ready', initval=1)
    txd = m.OutputReg('txd', initval=1)

    fsm = FSM(m, 'fsm', clk, rst)

    mem = m.TmpReg(9, initval=0)
    waitcount = m.TmpReg(int(math.log(waitnum, 2)) + 1, initval=0)

    fsm(
        waitcount(waitnum - 1),
        txd(1),
        mem(vtypes.Cat(din, vtypes.Int(0, 1)))
    )

    fsm.If(enable)(
        ready(0)
    )

    fsm.Then().goto_next()

    for i in range(10):
        fsm.If(waitcount > 0)(
            waitcount.dec()
        ).Else(
            txd(mem[0]),
            mem(vtypes.Cat(vtypes.Int(1, 1), mem[1:9])),
            waitcount(waitnum - 1)
        )
        fsm.Then().goto_next()

    fsm(
        ready(1)
    )

    fsm.goto_init()

    fsm.make_always()

    return m


def mkUartRx(baudrate=19200, clockfreq=100 * 1000 * 1000):
    m = Module("UartRx")
    waitnum = int(clockfreq / baudrate)

    clk = m.Input('CLK')
    rst = m.Input('RST')

    rxd = m.Input('rxd')
    dout = m.OutputReg('dout', 8, initval=0)
    valid = m.OutputReg('valid', initval=0)

    fsm = FSM(m, 'fsm', clk, rst)

    mem = m.TmpReg(9, initval=0)
    waitcount = m.TmpReg(int(math.log(waitnum, 2)) + 1, initval=0)

    fsm(
        valid(0),
        waitcount(int(waitnum / 2) - 1),
        mem(vtypes.Cat(rxd, mem[1:9]))
    )

    fsm.If(rxd == 0).goto_next()

    for i in range(10):
        if i == 0:  # check the start bit again
            fsm.If(vtypes.Ands(waitcount == 1, rxd != 0)).goto_init()

        fsm.If(waitcount > 0)(
            waitcount.dec()
        ).Else(
            mem(vtypes.Cat(rxd, mem[1:9])),
            waitcount(waitnum - 1)
        )
        fsm.Then().goto_next()

    fsm(
        valid(1),
        dout(mem[0:9])
    )

    fsm.goto_init()

    fsm.make_always()

    return m



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import collections

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *

def mkThread():
    filename = os.path.dirname(os.path.abspath(__file__)) + '/ctrl_thread.v'
    modules = from_verilog.read_verilog_module(filename)
    return modules

if __name__ == '__main__':
    modules = mkThread()
    verilog = ''.join([ m.to_verilog() for m in modules.values() ])
    print(verilog)



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
import math

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from veriloggen import *

def mkUartTx(baudrate=19200, clockfreq=100*1000*1000):
    m = Module("UartTx")
    waitnum = int(clockfreq / baudrate)
    
    clk = m.Input('CLK')
    rst = m.Input('RST')

    din = m.Input('din', 8)
    enable = m.Input('enable')
    ready = m.OutputReg('ready', initval=1)
    txd = m.OutputReg('txd', initval=1)

    fsm = FSM(m, 'fsm', clk, rst)

    mem = m.TmpReg(9, initval=0)
    waitcount = m.TmpReg(int(math.log(waitnum, 2)) + 1, initval=0)

    fsm(
        waitcount(waitnum-1),
        txd(1),
        mem(Cat(din, Int(0, 1)))
    )
    
    fsm.If(enable)(
        ready(0)
    )

    fsm.Then().goto_next()

    for i in range(10):
        fsm.If(waitcount>0)(
            waitcount.dec()
        ).Else(
            txd(mem[0]),
            mem(Cat(Int(1, 1), mem[1:9])),
            waitcount(waitnum-1)
        )
        fsm.Then().goto_next()

    fsm(
        ready(1)
    )
    
    fsm.goto_init()

    fsm.make_always()
    
    return m

def mkUartRx(baudrate=19200, clockfreq=100*1000*1000):
    m = Module("UartRx")
    waitnum = int(clockfreq / baudrate)
    
    clk = m.Input('CLK')
    rst = m.Input('RST')

    rxd = m.Input('rxd')
    dout = m.OutputReg('dout', 8, initval=0)
    valid = m.OutputReg('valid', initval=0)

    fsm = FSM(m, 'fsm', clk, rst)

    mem = m.TmpReg(9, initval=0)
    waitcount = m.TmpReg(int(math.log(waitnum, 2)) + 1, initval=0)

    fsm(
        valid(0),
        waitcount(int(waitnum/2)-1),
        mem(Cat(rxd, mem[1:9]))
    )
    
    fsm.If(rxd==0).goto_next()

    for i in range(10):
        if i == 0: # check the start bit again
            fsm.If(Ands(waitcount==1, rxd!=0)).goto_init()
            
        fsm.If(waitcount>0)(
            waitcount.dec()
        ).Else(
            mem(Cat(rxd, mem[1:9])),
            waitcount(waitnum-1)
        )
        fsm.Then().goto_next()
        
    fsm(
        valid(1),
        dout(mem[0:9])
    )
    
    fsm.goto_init()

    fsm.make_always()

    return m
    
def mkTest(baudrate=10*1000*1000):
    m = Module('test')
    
    mtx = mkUartTx(baudrate)
    mrx = mkUartRx(baudrate)

    clk = m.Reg('CLK')
    rst = m.Reg('RST')

    din = m.Reg('din', 8)
    enable = m.Reg('enable')
    ready = m.Wire('ready')
    txd = m.Wire('txd')

    dout = m.Wire('dout', 8)
    valid = m.Wire('valid')
    rxd = m.Wire('rxd')

    rxd.assign(txd)

    txfsm = FSM(m, 'txfsm', clk, rst)
    count = m.TmpReg(32, initval=0)

    send_data_list = (0x55, 0x33, 0x00, 0xff)

    for send_data in send_data_list:
        txfsm.add( din(send_data), enable(1), cond=ready)
        txfsm.goto_next(cond=ready)
        txfsm.add( enable(0) )
        txfsm.goto_next()
    
    txfsm.make_always()
    
    
    rxfsm = FSM(m, 'rxfsm', clk, rst)
    for send_data in send_data_list:
        check = Mux(send_data == dout, "OK", "NG")
        rxfsm.add(Systask('display', "din=%02x dout=%02x %s", Int(send_data, width=8), dout, check), cond=valid)
        rxfsm.goto_next(cond=valid)

    rxfsm.make_always()
    
    
    imtx = m.Instance(mtx, 'mtx',
                      params=m.connect_params(mtx),
                      ports=m.connect_ports(mtx))
    imrx = m.Instance(mrx, 'mrx',
                      params=m.connect_params(mrx),
                      ports=m.connect_ports(mrx))

    simulation.setup_waveform(m, imtx, imrx, m.get_vars())
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    nclk = simulation.next_clock
    
    init.add(
        Delay(1000 * 100),
        Systask('finish'),
    )

    return m

if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    # run simulator (Icarus Verilog)
    sim = simulation.Simulator(test)
    rslt = sim.run() # display=False
    #rslt = sim.run(display=True)
    print(rslt)

    # launch waveform viewer (GTKwave)
    #sim.view_waveform() # background=False
    #sim.view_waveform(background=True)



from __future__ import absolute_import
from __future__ import print_function

from .thread import reset, embed_thread, Thread, TmpThread
from .pool import ThreadPool, to_thread_pool
from .stream import Stream, TmpStream

from .ttypes import __intrinsics__
from .ttypes import *

from .ram import *
from .fifo import *
from .axi import *

from . import fixed



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import stream_reduceadd_valid

expected_verilog = """
module test;

  reg CLK;
  reg RST;
  reg ivalid;
  wire ovalid;
  reg signed [32-1:0] xdata;
  reg signed [32-1:0] ydata;
  wire signed [32-1:0] zdata;
  wire [1-1:0] vdata;

  main
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .ivalid(ivalid),
    .ovalid(ovalid),
    .xdata(xdata),
    .ydata(ydata),
    .zdata(zdata),
    .vdata(vdata)
  );

  reg reset_done;

  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    reset_done = 0;
    xdata = 0;
    ydata = 0;
    ivalid = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #1000;
    reset_done = 1;
    @(posedge CLK);
    #1;
    #10000;
    $finish;
  end

  reg [32-1:0] send_fsm;
  localparam send_fsm_init = 0;
  reg [32-1:0] send_count;
  reg [32-1:0] _d1_send_fsm;
  reg _send_fsm_cond_2_0_1;
  reg _send_fsm_cond_3_1_1;
  reg [32-1:0] recv_fsm;
  localparam recv_fsm_init = 0;
  reg [32-1:0] recv_count;
  localparam send_fsm_1 = 1;
  localparam send_fsm_2 = 2;
  localparam send_fsm_3 = 3;
  localparam send_fsm_4 = 4;

  always @(posedge CLK) begin
    if(RST) begin
      send_fsm <= send_fsm_init;
      _d1_send_fsm <= send_fsm_init;
      send_count <= 0;
      _send_fsm_cond_2_0_1 <= 0;
      _send_fsm_cond_3_1_1 <= 0;
    end else begin
      _d1_send_fsm <= send_fsm;
      case(_d1_send_fsm)
        send_fsm_2: begin
          if(_send_fsm_cond_2_0_1) begin
            $display("xdata=%d", xdata);
            $display("ydata=%d", ydata);
          end 
        end
        send_fsm_3: begin
          if(_send_fsm_cond_3_1_1) begin
            $display("xdata=%d", xdata);
            $display("ydata=%d", ydata);
          end 
        end
      endcase
      case(send_fsm)
        send_fsm_init: begin
          if(reset_done) begin
            send_fsm <= send_fsm_1;
          end 
        end
        send_fsm_1: begin
          ivalid <= 0;
          send_count <= send_count + 1;
          if(send_count == 10) begin
            send_count <= 0;
          end 
          if(send_count == 10) begin
            send_fsm <= send_fsm_2;
          end 
        end
        send_fsm_2: begin
          xdata <= 0;
          ydata <= 0;
          ivalid <= 1;
          send_count <= send_count + 1;
          _send_fsm_cond_2_0_1 <= 1;
          send_fsm <= send_fsm_3;
        end
        send_fsm_3: begin
          xdata <= xdata + 1;
          ydata <= ydata + 2;
          ivalid <= 1;
          send_count <= send_count + 1;
          _send_fsm_cond_3_1_1 <= 1;
          if(send_count == 65) begin
            ivalid <= 0;
          end 
          if(send_count == 65) begin
            send_fsm <= send_fsm_4;
          end 
        end
      endcase
    end
  end

  localparam recv_fsm_1 = 1;

  always @(posedge CLK) begin
    if(RST) begin
      recv_fsm <= recv_fsm_init;
      recv_count <= 0;
    end else begin
      case(recv_fsm)
        recv_fsm_init: begin
          if(reset_done) begin
            recv_fsm <= recv_fsm_1;
          end 
        end
        recv_fsm_1: begin
          if(ovalid && vdata) begin
            $display("zdata=%d", zdata);
            recv_count <= recv_count + 1;
          end 
        end
      endcase
    end
  end


endmodule



module main
(
  input CLK,
  input RST,
  input ivalid,
  output ovalid,
  input signed [32-1:0] xdata,
  input signed [32-1:0] ydata,
  output signed [32-1:0] zdata,
  output [1-1:0] vdata
);

  reg _ivalid_0;
  reg _ivalid_1;
  assign ovalid = _ivalid_1;
  reg signed [32-1:0] _plus_data_2;
  reg signed [32-1:0] _reduceadd_data_5;
  reg [6-1:0] _reduceadd_count_5;
  reg [1-1:0] _pulse_data_8;
  reg [6-1:0] _pulse_count_8;
  assign zdata = _reduceadd_data_5;
  assign vdata = _pulse_data_8;

  always @(posedge CLK) begin
    if(RST) begin
      _ivalid_0 <= 0;
      _ivalid_1 <= 0;
      _plus_data_2 <= 0;
      _reduceadd_data_5 <= 1'sd0;
      _reduceadd_count_5 <= 0;
      _pulse_data_8 <= 1'sd0;
      _pulse_count_8 <= 0;
    end else begin
      _ivalid_0 <= ivalid;
      _ivalid_1 <= _ivalid_0;
      _plus_data_2 <= xdata + ydata;
      if(_ivalid_0) begin
        _reduceadd_data_5 <= _reduceadd_data_5 + _plus_data_2;
      end 
      if(_ivalid_0) begin
        _reduceadd_count_5 <= (_reduceadd_count_5 >= 5'sd8 - 1)? 0 : _reduceadd_count_5 + 1;
      end 
      if(_ivalid_0 && (_reduceadd_count_5 == 0)) begin
        _reduceadd_data_5 <= 1'sd0 + _plus_data_2;
      end 
      if(_ivalid_0) begin
        _pulse_data_8 <= _pulse_count_8 >= 5'sd8 - 1;
      end 
      if(_ivalid_0) begin
        _pulse_count_8 <= (_pulse_count_8 >= 5'sd8 - 1)? 0 : _pulse_count_8 + 1;
      end 
      if(_ivalid_0 && (_pulse_count_8 == 0)) begin
        _pulse_data_8 <= _pulse_count_8 >= 5'sd8 - 1;
      end 
    end
  end


endmodule
"""


def test():
    veriloggen.reset()
    test_module = stream_reduceadd_valid.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import os
import sys

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from veriloggen import *

def mkRegChain(length=120, width=8):
    m = Module("reg_chain")

    clk = m.Input('CLK')
    rst = m.Input('RST')
    sw = m.Input('sw', 8)
    dout = m.Output('dout', width)

    seq = Seq(m, 'seq', clk, rst)

    update_cond_value = m.TmpReg(3, initval=0)
    seq( update_cond_value(sw[0:3]) )

    area_size = m.TmpReg(2, initval=0)
    seq( area_size(sw[3:5]) )
    
    count = m.TmpReg(2, initval=0)
    seq( count.inc() )
    
    update_cond = m.TmpReg(initval=0)
    seq( update_cond(count < update_cond_value) )

    orig = m.TmpReg(width, initval=0)
    prev = orig

    regs = []
    
    for i in range(length):
        area_id = i // (length // 4)
        r = m.TmpReg(width, initval=0)
        regs.append(r)
        seq.If(AndList(update_cond, area_id <= area_size))( r(prev + 1) )
        prev = r

    seq.If(AndList(update_cond, area_size==0))( orig(regs[1*length//4-1] + 3) )
    seq.If(AndList(update_cond, area_size==1))( orig(regs[2*length//4-1] + 2) )
    seq.If(AndList(update_cond, area_size==2))( orig(regs[3*length//4-1] + 1) )
    seq.If(AndList(update_cond, area_size==3))( orig(regs[4*length//4-1] + 0) )

    seq.make_always()
    
    m.Assign( dout(orig) )

    return m

def mkTest(length=120, width=8):
    m = Module('test')
    
    main = mkRegChain(length, width)
    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)
    
    clk = ports['CLK']
    rst = ports['RST']
    sw = ports['sw']
    dout = ports['dout']

    fsm = FSM(m, 'fsm', clk, rst)
    count = m.TmpReg(32, initval=0)
    
    fsm( sw((3 << 3) | 4) )
    fsm( count.inc() )
    fsm.If(count==2000)( count(0) )
    fsm.goto_next(count==2000)

    fsm( sw((2 << 3) | 4) )
    fsm( count.inc() )
    fsm.If(count==2000)( count(0) )
    fsm.goto_next(count==2000)

    fsm( sw((1 << 3) | 4) )
    fsm( count.inc() )
    fsm.If(count==2000)( count(0) )
    fsm.goto_next(count==2000)

    fsm( sw((0 << 3) | 4) )
    fsm( count.inc() )
    fsm.If(count==2000)( count(0) )
    fsm.goto_next(count==2000)

    fsm( sw((2 << 3) | 3) )
    fsm( count.inc() )
    fsm.If(count==2000)( count(0) )
    fsm.goto_next(count==2000)

    fsm( sw((2 << 3) | 2) )
    fsm( count.inc() )
    fsm.If(count==2000)( count(0) )
    fsm.goto_next(count==2000)

    fsm( sw((2 << 3) | 1) )
    fsm( count.inc() )
    fsm.If(count==2000)( count(0) )
    fsm.goto_next(count==2000)

    fsm( sw((2 << 3) | 0) )
    fsm( count.inc() )
    fsm.If(count==2000)( count(0) )
    fsm.goto_next(count==2000)

    fsm.make_always()
    
    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))

    simulation.setup_waveform(m, uut, m.get_vars())
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    nclk = simulation.next_clock
    
    init.add(
        sw(0),
        Delay(1000 * 200),
        Systask('finish'),
    )

    return m
    
if __name__ == '__main__':
    main = mkRegChain(length=120)
    verilog = main.to_verilog('tmp.v')
    print(verilog)

    #test = mkTest()
    #verilog = test.to_verilog('tmp.v')
    #print(verilog)

    # run simulator (Icarus Verilog)
    #sim = simulation.Simulator(test)
    #rslt = sim.run() # display=False
    ##rslt = sim.run(display=True)
    #print(rslt)

    # launch waveform viewer (GTKwave)
    #sim.view_waveform() # background=False
    #sim.view_waveform(background=True)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from veriloggen import *

def mkLed():
    m = Module('blinkled')
    width = m.Parameter('WIDTH', 8)
    clk = m.Input('CLK')
    rst = m.Input('RST')

    # function to add an LED port
    def add_led(postfix, limit=1024):
        led = m.OutputReg('LED'+postfix, width)
        count = m.Reg('count'+postfix, 32)

        m.Always(Posedge(clk))(
            If(rst)(
                count(0)
            ).Else(
                If(count == limit - 1)(
                    count(0)
                ).Else(
                    count(count + 1)
                )
            ))
    
        m.Always(Posedge(clk))(
            If(rst)(
                led(0)
            ).Else(
                If(count == limit - 1)(
                    led(led + 1)
                )
            ))

    # call 'add_led' to add LED ports
    for i in range(4):
        add_led('_' + str(i), limit=i*10 + 10)
    
    return m

if __name__ == '__main__':
    led = mkLed()
    verilog = led.to_verilog()
    print(verilog)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import embeddednumeric

expected_verilog = """

module blinkled #
(
  parameter WIDTH = 8
)
(
  input CLK,
  input RST,
  output reg [WIDTH-1:0] LED
);

  reg [32-1:0] count;

  always @(posedge CLK) begin
    if(RST) begin
      count <= 0;
    end else begin
      if(count == 1023) begin
        count <= 0;
      end else begin
        count <= count + 1;
      end
    end
  end


  always @(posedge CLK) begin
    if(RST) begin
      LED <= 0;
    end else begin
      if(count == 1023) begin
        LED <= LED + 1;
      end 
    end
  end


endmodule

"""


def test():
    veriloggen.reset()
    test_module = embeddednumeric.mkLed()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    #parser = VerilogParser()
    #expected_ast = parser.parse(expected_verilog)
    #codegen = ASTCodeGenerator()
    #expected_code = codegen.visit(expected_ast)
    expected_code = expected_verilog

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import thread_ram_own_mutex

expected_verilog = """
module test;

  reg CLK;
  reg RST;
  wire [8-1:0] LED;

  blinkled
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .LED(LED)
  );


  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #100000;
    $finish;
  end


endmodule



module blinkled
(
  input CLK,
  input RST,
  output reg [8-1:0] LED
);

  reg [10-1:0] myram_0_addr;
  wire [32-1:0] myram_0_rdata;
  reg [32-1:0] myram_0_wdata;
  reg myram_0_wenable;

  myram
  inst_myram
  (
    .CLK(CLK),
    .myram_0_addr(myram_0_addr),
    .myram_0_rdata(myram_0_rdata),
    .myram_0_wdata(myram_0_wdata),
    .myram_0_wenable(myram_0_wenable)
  );

  reg [8-1:0] _th_myfunc_start;
  reg [32-1:0] th_blink;
  localparam th_blink_init = 0;
  reg signed [32-1:0] _th_blink_size_0;
  reg signed [32-1:0] _th_blink_i_1;
  reg _myram_cond_0_1;
  reg signed [32-1:0] _th_blink_tid_2;
  reg [32-1:0] th_myfunc_0;
  localparam th_myfunc_0_init = 0;
  reg [32-1:0] th_myfunc_1;
  localparam th_myfunc_1_init = 0;
  reg [32-1:0] th_myfunc_2;
  localparam th_myfunc_2_init = 0;
  reg [32-1:0] th_myfunc_3;
  localparam th_myfunc_3_init = 0;
  reg [32-1:0] th_myfunc_4;
  localparam th_myfunc_4_init = 0;
  reg [32-1:0] th_myfunc_5;
  localparam th_myfunc_5_init = 0;
  reg [32-1:0] th_myfunc_6;
  localparam th_myfunc_6_init = 0;
  reg [32-1:0] th_myfunc_7;
  localparam th_myfunc_7_init = 0;
  reg _th_myfunc_0_called;
  reg signed [32-1:0] _th_myfunc_0_tid_3;
  reg signed [32-1:0] _th_myfunc_0_size_4;
  reg signed [32-1:0] _th_myfunc_0_tid_5;
  reg signed [32-1:0] _th_myfunc_0_size_6;
  reg __myram_mutex_lock_reg;
  reg [32-1:0] __myram_mutex_lock_id;
  reg signed [32-1:0] _th_myfunc_0_i_7;
  reg _tmp_0;
  reg _myram_cond_1_1;
  reg _myram_cond_2_1;
  reg _myram_cond_2_2;
  reg signed [32-1:0] _tmp_1;
  reg signed [32-1:0] _th_myfunc_0_read_data_8;
  reg signed [32-1:0] _th_myfunc_0_write_data_9;
  reg _myram_cond_3_1;
  reg _th_myfunc_1_called;
  reg signed [32-1:0] _th_myfunc_1_tid_10;
  reg signed [32-1:0] _th_myfunc_1_size_11;
  reg signed [32-1:0] _th_myfunc_1_tid_12;
  reg signed [32-1:0] _th_myfunc_1_size_13;
  reg signed [32-1:0] _th_myfunc_1_i_14;
  reg _tmp_2;
  reg _myram_cond_4_1;
  reg _myram_cond_5_1;
  reg _myram_cond_5_2;
  reg signed [32-1:0] _tmp_3;
  reg signed [32-1:0] _th_myfunc_1_read_data_15;
  reg signed [32-1:0] _th_myfunc_1_write_data_16;
  reg _myram_cond_6_1;
  reg _th_myfunc_2_called;
  reg signed [32-1:0] _th_myfunc_2_tid_17;
  reg signed [32-1:0] _th_myfunc_2_size_18;
  reg signed [32-1:0] _th_myfunc_2_tid_19;
  reg signed [32-1:0] _th_myfunc_2_size_20;
  reg signed [32-1:0] _th_myfunc_2_i_21;
  reg _tmp_4;
  reg _myram_cond_7_1;
  reg _myram_cond_8_1;
  reg _myram_cond_8_2;
  reg signed [32-1:0] _tmp_5;
  reg signed [32-1:0] _th_myfunc_2_read_data_22;
  reg signed [32-1:0] _th_myfunc_2_write_data_23;
  reg _myram_cond_9_1;
  reg _th_myfunc_3_called;
  reg signed [32-1:0] _th_myfunc_3_tid_24;
  reg signed [32-1:0] _th_myfunc_3_size_25;
  reg signed [32-1:0] _th_myfunc_3_tid_26;
  reg signed [32-1:0] _th_myfunc_3_size_27;
  reg signed [32-1:0] _th_myfunc_3_i_28;
  reg _tmp_6;
  reg _myram_cond_10_1;
  reg _myram_cond_11_1;
  reg _myram_cond_11_2;
  reg signed [32-1:0] _tmp_7;
  reg signed [32-1:0] _th_myfunc_3_read_data_29;
  reg signed [32-1:0] _th_myfunc_3_write_data_30;
  reg _myram_cond_12_1;
  reg _th_myfunc_4_called;
  reg signed [32-1:0] _th_myfunc_4_tid_31;
  reg signed [32-1:0] _th_myfunc_4_size_32;
  reg signed [32-1:0] _th_myfunc_4_tid_33;
  reg signed [32-1:0] _th_myfunc_4_size_34;
  reg signed [32-1:0] _th_myfunc_4_i_35;
  reg _tmp_8;
  reg _myram_cond_13_1;
  reg _myram_cond_14_1;
  reg _myram_cond_14_2;
  reg signed [32-1:0] _tmp_9;
  reg signed [32-1:0] _th_myfunc_4_read_data_36;
  reg signed [32-1:0] _th_myfunc_4_write_data_37;
  reg _myram_cond_15_1;
  reg _th_myfunc_5_called;
  reg signed [32-1:0] _th_myfunc_5_tid_38;
  reg signed [32-1:0] _th_myfunc_5_size_39;
  reg signed [32-1:0] _th_myfunc_5_tid_40;
  reg signed [32-1:0] _th_myfunc_5_size_41;
  reg signed [32-1:0] _th_myfunc_5_i_42;
  reg _tmp_10;
  reg _myram_cond_16_1;
  reg _myram_cond_17_1;
  reg _myram_cond_17_2;
  reg signed [32-1:0] _tmp_11;
  reg signed [32-1:0] _th_myfunc_5_read_data_43;
  reg signed [32-1:0] _th_myfunc_5_write_data_44;
  reg _myram_cond_18_1;
  reg _th_myfunc_6_called;
  reg signed [32-1:0] _th_myfunc_6_tid_45;
  reg signed [32-1:0] _th_myfunc_6_size_46;
  reg signed [32-1:0] _th_myfunc_6_tid_47;
  reg signed [32-1:0] _th_myfunc_6_size_48;
  reg signed [32-1:0] _th_myfunc_6_i_49;
  reg _tmp_12;
  reg _myram_cond_19_1;
  reg _myram_cond_20_1;
  reg _myram_cond_20_2;
  reg signed [32-1:0] _tmp_13;
  reg signed [32-1:0] _th_myfunc_6_read_data_50;
  reg signed [32-1:0] _th_myfunc_6_write_data_51;
  reg _myram_cond_21_1;
  reg _th_myfunc_7_called;
  reg signed [32-1:0] _th_myfunc_7_tid_52;
  reg signed [32-1:0] _th_myfunc_7_size_53;
  reg signed [32-1:0] _th_myfunc_7_tid_54;
  reg signed [32-1:0] _th_myfunc_7_size_55;
  reg signed [32-1:0] _th_myfunc_7_i_56;
  reg _tmp_14;
  reg _myram_cond_22_1;
  reg _myram_cond_23_1;
  reg _myram_cond_23_2;
  reg signed [32-1:0] _tmp_15;
  reg signed [32-1:0] _th_myfunc_7_read_data_57;
  reg signed [32-1:0] _th_myfunc_7_write_data_58;
  reg _myram_cond_24_1;
  reg _tmp_16;
  reg _myram_cond_25_1;
  reg _myram_cond_26_1;
  reg _myram_cond_26_2;
  reg signed [32-1:0] _tmp_17;
  reg signed [32-1:0] _th_blink_read_data_59;

  always @(posedge CLK) begin
    if(RST) begin
      myram_0_addr <= 0;
      myram_0_wdata <= 0;
      myram_0_wenable <= 0;
      _myram_cond_0_1 <= 0;
      _myram_cond_1_1 <= 0;
      _tmp_0 <= 0;
      _myram_cond_2_1 <= 0;
      _myram_cond_2_2 <= 0;
      _myram_cond_3_1 <= 0;
      _myram_cond_4_1 <= 0;
      _tmp_2 <= 0;
      _myram_cond_5_1 <= 0;
      _myram_cond_5_2 <= 0;
      _myram_cond_6_1 <= 0;
      _myram_cond_7_1 <= 0;
      _tmp_4 <= 0;
      _myram_cond_8_1 <= 0;
      _myram_cond_8_2 <= 0;
      _myram_cond_9_1 <= 0;
      _myram_cond_10_1 <= 0;
      _tmp_6 <= 0;
      _myram_cond_11_1 <= 0;
      _myram_cond_11_2 <= 0;
      _myram_cond_12_1 <= 0;
      _myram_cond_13_1 <= 0;
      _tmp_8 <= 0;
      _myram_cond_14_1 <= 0;
      _myram_cond_14_2 <= 0;
      _myram_cond_15_1 <= 0;
      _myram_cond_16_1 <= 0;
      _tmp_10 <= 0;
      _myram_cond_17_1 <= 0;
      _myram_cond_17_2 <= 0;
      _myram_cond_18_1 <= 0;
      _myram_cond_19_1 <= 0;
      _tmp_12 <= 0;
      _myram_cond_20_1 <= 0;
      _myram_cond_20_2 <= 0;
      _myram_cond_21_1 <= 0;
      _myram_cond_22_1 <= 0;
      _tmp_14 <= 0;
      _myram_cond_23_1 <= 0;
      _myram_cond_23_2 <= 0;
      _myram_cond_24_1 <= 0;
      _myram_cond_25_1 <= 0;
      _tmp_16 <= 0;
      _myram_cond_26_1 <= 0;
      _myram_cond_26_2 <= 0;
    end else begin
      if(_myram_cond_2_2) begin
        _tmp_0 <= 0;
      end 
      if(_myram_cond_5_2) begin
        _tmp_2 <= 0;
      end 
      if(_myram_cond_8_2) begin
        _tmp_4 <= 0;
      end 
      if(_myram_cond_11_2) begin
        _tmp_6 <= 0;
      end 
      if(_myram_cond_14_2) begin
        _tmp_8 <= 0;
      end 
      if(_myram_cond_17_2) begin
        _tmp_10 <= 0;
      end 
      if(_myram_cond_20_2) begin
        _tmp_12 <= 0;
      end 
      if(_myram_cond_23_2) begin
        _tmp_14 <= 0;
      end 
      if(_myram_cond_26_2) begin
        _tmp_16 <= 0;
      end 
      if(_myram_cond_0_1) begin
        myram_0_wenable <= 0;
      end 
      if(_myram_cond_1_1) begin
        _tmp_0 <= 1;
      end 
      _myram_cond_2_2 <= _myram_cond_2_1;
      if(_myram_cond_3_1) begin
        myram_0_wenable <= 0;
      end 
      if(_myram_cond_4_1) begin
        _tmp_2 <= 1;
      end 
      _myram_cond_5_2 <= _myram_cond_5_1;
      if(_myram_cond_6_1) begin
        myram_0_wenable <= 0;
      end 
      if(_myram_cond_7_1) begin
        _tmp_4 <= 1;
      end 
      _myram_cond_8_2 <= _myram_cond_8_1;
      if(_myram_cond_9_1) begin
        myram_0_wenable <= 0;
      end 
      if(_myram_cond_10_1) begin
        _tmp_6 <= 1;
      end 
      _myram_cond_11_2 <= _myram_cond_11_1;
      if(_myram_cond_12_1) begin
        myram_0_wenable <= 0;
      end 
      if(_myram_cond_13_1) begin
        _tmp_8 <= 1;
      end 
      _myram_cond_14_2 <= _myram_cond_14_1;
      if(_myram_cond_15_1) begin
        myram_0_wenable <= 0;
      end 
      if(_myram_cond_16_1) begin
        _tmp_10 <= 1;
      end 
      _myram_cond_17_2 <= _myram_cond_17_1;
      if(_myram_cond_18_1) begin
        myram_0_wenable <= 0;
      end 
      if(_myram_cond_19_1) begin
        _tmp_12 <= 1;
      end 
      _myram_cond_20_2 <= _myram_cond_20_1;
      if(_myram_cond_21_1) begin
        myram_0_wenable <= 0;
      end 
      if(_myram_cond_22_1) begin
        _tmp_14 <= 1;
      end 
      _myram_cond_23_2 <= _myram_cond_23_1;
      if(_myram_cond_24_1) begin
        myram_0_wenable <= 0;
      end 
      if(_myram_cond_25_1) begin
        _tmp_16 <= 1;
      end 
      _myram_cond_26_2 <= _myram_cond_26_1;
      if(th_blink == 4) begin
        myram_0_addr <= _th_blink_i_1;
        myram_0_wdata <= 0;
        myram_0_wenable <= 1;
      end 
      _myram_cond_0_1 <= th_blink == 4;
      if(th_myfunc_0 == 7) begin
        myram_0_addr <= _th_myfunc_0_i_7;
      end 
      _myram_cond_1_1 <= th_myfunc_0 == 7;
      _myram_cond_2_1 <= th_myfunc_0 == 7;
      if(th_myfunc_0 == 10) begin
        myram_0_addr <= _th_myfunc_0_i_7;
        myram_0_wdata <= _th_myfunc_0_write_data_9;
        myram_0_wenable <= 1;
      end 
      _myram_cond_3_1 <= th_myfunc_0 == 10;
      if(th_myfunc_1 == 7) begin
        myram_0_addr <= _th_myfunc_1_i_14;
      end 
      _myram_cond_4_1 <= th_myfunc_1 == 7;
      _myram_cond_5_1 <= th_myfunc_1 == 7;
      if(th_myfunc_1 == 10) begin
        myram_0_addr <= _th_myfunc_1_i_14;
        myram_0_wdata <= _th_myfunc_1_write_data_16;
        myram_0_wenable <= 1;
      end 
      _myram_cond_6_1 <= th_myfunc_1 == 10;
      if(th_myfunc_2 == 7) begin
        myram_0_addr <= _th_myfunc_2_i_21;
      end 
      _myram_cond_7_1 <= th_myfunc_2 == 7;
      _myram_cond_8_1 <= th_myfunc_2 == 7;
      if(th_myfunc_2 == 10) begin
        myram_0_addr <= _th_myfunc_2_i_21;
        myram_0_wdata <= _th_myfunc_2_write_data_23;
        myram_0_wenable <= 1;
      end 
      _myram_cond_9_1 <= th_myfunc_2 == 10;
      if(th_myfunc_3 == 7) begin
        myram_0_addr <= _th_myfunc_3_i_28;
      end 
      _myram_cond_10_1 <= th_myfunc_3 == 7;
      _myram_cond_11_1 <= th_myfunc_3 == 7;
      if(th_myfunc_3 == 10) begin
        myram_0_addr <= _th_myfunc_3_i_28;
        myram_0_wdata <= _th_myfunc_3_write_data_30;
        myram_0_wenable <= 1;
      end 
      _myram_cond_12_1 <= th_myfunc_3 == 10;
      if(th_myfunc_4 == 7) begin
        myram_0_addr <= _th_myfunc_4_i_35;
      end 
      _myram_cond_13_1 <= th_myfunc_4 == 7;
      _myram_cond_14_1 <= th_myfunc_4 == 7;
      if(th_myfunc_4 == 10) begin
        myram_0_addr <= _th_myfunc_4_i_35;
        myram_0_wdata <= _th_myfunc_4_write_data_37;
        myram_0_wenable <= 1;
      end 
      _myram_cond_15_1 <= th_myfunc_4 == 10;
      if(th_myfunc_5 == 7) begin
        myram_0_addr <= _th_myfunc_5_i_42;
      end 
      _myram_cond_16_1 <= th_myfunc_5 == 7;
      _myram_cond_17_1 <= th_myfunc_5 == 7;
      if(th_myfunc_5 == 10) begin
        myram_0_addr <= _th_myfunc_5_i_42;
        myram_0_wdata <= _th_myfunc_5_write_data_44;
        myram_0_wenable <= 1;
      end 
      _myram_cond_18_1 <= th_myfunc_5 == 10;
      if(th_myfunc_6 == 7) begin
        myram_0_addr <= _th_myfunc_6_i_49;
      end 
      _myram_cond_19_1 <= th_myfunc_6 == 7;
      _myram_cond_20_1 <= th_myfunc_6 == 7;
      if(th_myfunc_6 == 10) begin
        myram_0_addr <= _th_myfunc_6_i_49;
        myram_0_wdata <= _th_myfunc_6_write_data_51;
        myram_0_wenable <= 1;
      end 
      _myram_cond_21_1 <= th_myfunc_6 == 10;
      if(th_myfunc_7 == 7) begin
        myram_0_addr <= _th_myfunc_7_i_56;
      end 
      _myram_cond_22_1 <= th_myfunc_7 == 7;
      _myram_cond_23_1 <= th_myfunc_7 == 7;
      if(th_myfunc_7 == 10) begin
        myram_0_addr <= _th_myfunc_7_i_56;
        myram_0_wdata <= _th_myfunc_7_write_data_58;
        myram_0_wenable <= 1;
      end 
      _myram_cond_24_1 <= th_myfunc_7 == 10;
      if(th_blink == 18) begin
        myram_0_addr <= _th_blink_i_1;
      end 
      _myram_cond_25_1 <= th_blink == 18;
      _myram_cond_26_1 <= th_blink == 18;
    end
  end

  localparam th_blink_1 = 1;
  localparam th_blink_2 = 2;
  localparam th_blink_3 = 3;
  localparam th_blink_4 = 4;
  localparam th_blink_5 = 5;
  localparam th_blink_6 = 6;
  localparam th_blink_7 = 7;
  localparam th_blink_8 = 8;
  localparam th_blink_9 = 9;
  localparam th_blink_10 = 10;
  localparam th_blink_11 = 11;
  localparam th_blink_12 = 12;
  localparam th_blink_13 = 13;
  localparam th_blink_14 = 14;
  localparam th_blink_15 = 15;
  localparam th_blink_16 = 16;
  localparam th_blink_17 = 17;
  localparam th_blink_18 = 18;
  localparam th_blink_19 = 19;
  localparam th_blink_20 = 20;
  localparam th_blink_21 = 21;
  localparam th_blink_22 = 22;
  localparam th_blink_23 = 23;

  always @(posedge CLK) begin
    if(RST) begin
      th_blink <= th_blink_init;
      _th_blink_size_0 <= 0;
      _th_blink_i_1 <= 0;
      _th_blink_tid_2 <= 0;
      _th_myfunc_start[_th_blink_tid_2] <= (0 >> _th_blink_tid_2) & 1'd1;
      _tmp_17 <= 0;
      _th_blink_read_data_59 <= 0;
    end else begin
      case(th_blink)
        th_blink_init: begin
          th_blink <= th_blink_1;
        end
        th_blink_1: begin
          _th_blink_size_0 <= 16;
          th_blink <= th_blink_2;
        end
        th_blink_2: begin
          _th_blink_i_1 <= 0;
          th_blink <= th_blink_3;
        end
        th_blink_3: begin
          if(_th_blink_i_1 < _th_blink_size_0) begin
            th_blink <= th_blink_4;
          end else begin
            th_blink <= th_blink_6;
          end
        end
        th_blink_4: begin
          th_blink <= th_blink_5;
        end
        th_blink_5: begin
          _th_blink_i_1 <= _th_blink_i_1 + 1;
          th_blink <= th_blink_3;
        end
        th_blink_6: begin
          _th_blink_tid_2 <= 0;
          th_blink <= th_blink_7;
        end
        th_blink_7: begin
          if(_th_blink_tid_2 < 8) begin
            th_blink <= th_blink_8;
          end else begin
            th_blink <= th_blink_12;
          end
        end
        th_blink_8: begin
          _th_myfunc_start[_th_blink_tid_2] <= 1;
          th_blink <= th_blink_9;
        end
        th_blink_9: begin
          th_blink <= th_blink_10;
          th_blink <= th_blink_10;
          th_blink <= th_blink_10;
          th_blink <= th_blink_10;
          th_blink <= th_blink_10;
          th_blink <= th_blink_10;
          th_blink <= th_blink_10;
          th_blink <= th_blink_10;
        end
        th_blink_10: begin
          _th_myfunc_start[_th_blink_tid_2] <= 0;
          th_blink <= th_blink_11;
        end
        th_blink_11: begin
          _th_blink_tid_2 <= _th_blink_tid_2 + 1;
          th_blink <= th_blink_7;
        end
        th_blink_12: begin
          _th_blink_tid_2 <= 0;
          th_blink <= th_blink_13;
        end
        th_blink_13: begin
          if(_th_blink_tid_2 < 8) begin
            th_blink <= th_blink_14;
          end else begin
            th_blink <= th_blink_16;
          end
        end
        th_blink_14: begin
          if((_th_blink_tid_2 == 0)? th_myfunc_0 == 15 : 
          (_th_blink_tid_2 == 1)? th_myfunc_1 == 15 : 
          (_th_blink_tid_2 == 2)? th_myfunc_2 == 15 : 
          (_th_blink_tid_2 == 3)? th_myfunc_3 == 15 : 
          (_th_blink_tid_2 == 4)? th_myfunc_4 == 15 : 
          (_th_blink_tid_2 == 5)? th_myfunc_5 == 15 : 
          (_th_blink_tid_2 == 6)? th_myfunc_6 == 15 : 
          (_th_blink_tid_2 == 7)? th_myfunc_7 == 15 : 0) begin
            th_blink <= th_blink_15;
          end 
        end
        th_blink_15: begin
          _th_blink_tid_2 <= _th_blink_tid_2 + 1;
          th_blink <= th_blink_13;
        end
        th_blink_16: begin
          _th_blink_i_1 <= 0;
          th_blink <= th_blink_17;
        end
        th_blink_17: begin
          if(_th_blink_i_1 < _th_blink_size_0) begin
            th_blink <= th_blink_18;
          end else begin
            th_blink <= th_blink_23;
          end
        end
        th_blink_18: begin
          if(_tmp_16) begin
            _tmp_17 <= myram_0_rdata;
          end 
          if(_tmp_16) begin
            th_blink <= th_blink_19;
          end 
        end
        th_blink_19: begin
          _th_blink_read_data_59 <= _tmp_17;
          th_blink <= th_blink_20;
        end
        th_blink_20: begin
          LED <= _th_blink_read_data_59;
          th_blink <= th_blink_21;
        end
        th_blink_21: begin
          $display("result ram[%d] = %d", _th_blink_i_1, _th_blink_read_data_59);
          th_blink <= th_blink_22;
        end
        th_blink_22: begin
          _th_blink_i_1 <= _th_blink_i_1 + 1;
          th_blink <= th_blink_17;
        end
      endcase
    end
  end

  localparam th_myfunc_0_1 = 1;
  localparam th_myfunc_0_2 = 2;
  localparam th_myfunc_0_3 = 3;
  localparam th_myfunc_0_4 = 4;
  localparam th_myfunc_0_5 = 5;
  localparam th_myfunc_0_6 = 6;
  localparam th_myfunc_0_7 = 7;
  localparam th_myfunc_0_8 = 8;
  localparam th_myfunc_0_9 = 9;
  localparam th_myfunc_0_10 = 10;
  localparam th_myfunc_0_11 = 11;
  localparam th_myfunc_0_12 = 12;
  localparam th_myfunc_0_13 = 13;
  localparam th_myfunc_0_14 = 14;
  localparam th_myfunc_0_15 = 15;

  always @(posedge CLK) begin
    if(RST) begin
      th_myfunc_0 <= th_myfunc_0_init;
      _th_myfunc_0_called <= 0;
      _th_myfunc_0_tid_3 <= 0;
      _th_myfunc_0_size_4 <= 0;
      _th_myfunc_0_tid_5 <= 0;
      _th_myfunc_0_size_6 <= 0;
      _th_myfunc_0_i_7 <= 0;
      _tmp_1 <= 0;
      _th_myfunc_0_read_data_8 <= 0;
      _th_myfunc_0_write_data_9 <= 0;
    end else begin
      case(th_myfunc_0)
        th_myfunc_0_init: begin
          if(_th_myfunc_start[0] && (th_blink == 9)) begin
            _th_myfunc_0_called <= 1;
          end 
          if(_th_myfunc_start[0] && (th_blink == 9)) begin
            _th_myfunc_0_tid_3 <= _th_blink_tid_2;
          end 
          if(_th_myfunc_start[0] && (th_blink == 9)) begin
            _th_myfunc_0_size_4 <= _th_blink_size_0;
          end 
          if((th_blink == 9) && _th_myfunc_start[0]) begin
            th_myfunc_0 <= th_myfunc_0_1;
          end 
        end
        th_myfunc_0_1: begin
          _th_myfunc_0_tid_5 <= _th_myfunc_0_tid_3;
          _th_myfunc_0_size_6 <= _th_myfunc_0_size_4;
          th_myfunc_0 <= th_myfunc_0_2;
        end
        th_myfunc_0_2: begin
          if(!__myram_mutex_lock_reg || (__myram_mutex_lock_id == 0)) begin
            th_myfunc_0 <= th_myfunc_0_3;
          end 
        end
        th_myfunc_0_3: begin
          if(!(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 0))) begin
            th_myfunc_0 <= th_myfunc_0_2;
          end 
          if(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 0)) begin
            th_myfunc_0 <= th_myfunc_0_4;
          end 
        end
        th_myfunc_0_4: begin
          $display("Thread %d Lock", _th_myfunc_0_tid_5);
          th_myfunc_0 <= th_myfunc_0_5;
        end
        th_myfunc_0_5: begin
          _th_myfunc_0_i_7 <= 0;
          th_myfunc_0 <= th_myfunc_0_6;
        end
        th_myfunc_0_6: begin
          if(_th_myfunc_0_i_7 < _th_myfunc_0_size_6) begin
            th_myfunc_0 <= th_myfunc_0_7;
          end else begin
            th_myfunc_0 <= th_myfunc_0_13;
          end
        end
        th_myfunc_0_7: begin
          if(_tmp_0) begin
            _tmp_1 <= myram_0_rdata;
          end 
          if(_tmp_0) begin
            th_myfunc_0 <= th_myfunc_0_8;
          end 
        end
        th_myfunc_0_8: begin
          _th_myfunc_0_read_data_8 <= _tmp_1;
          th_myfunc_0 <= th_myfunc_0_9;
        end
        th_myfunc_0_9: begin
          _th_myfunc_0_write_data_9 <= _th_myfunc_0_read_data_8 + _th_myfunc_0_tid_5 + _th_myfunc_0_i_7;
          th_myfunc_0 <= th_myfunc_0_10;
        end
        th_myfunc_0_10: begin
          th_myfunc_0 <= th_myfunc_0_11;
        end
        th_myfunc_0_11: begin
          $display("Thread %d ram[%d] <- %d", _th_myfunc_0_tid_5, _th_myfunc_0_i_7, _th_myfunc_0_write_data_9);
          th_myfunc_0 <= th_myfunc_0_12;
        end
        th_myfunc_0_12: begin
          _th_myfunc_0_i_7 <= _th_myfunc_0_i_7 + 1;
          th_myfunc_0 <= th_myfunc_0_6;
        end
        th_myfunc_0_13: begin
          th_myfunc_0 <= th_myfunc_0_14;
        end
        th_myfunc_0_14: begin
          $display("Thread %d Unlock", _th_myfunc_0_tid_5);
          th_myfunc_0 <= th_myfunc_0_15;
        end
      endcase
    end
  end

  localparam th_myfunc_1_1 = 1;
  localparam th_myfunc_1_2 = 2;
  localparam th_myfunc_1_3 = 3;
  localparam th_myfunc_1_4 = 4;
  localparam th_myfunc_1_5 = 5;
  localparam th_myfunc_1_6 = 6;
  localparam th_myfunc_1_7 = 7;
  localparam th_myfunc_1_8 = 8;
  localparam th_myfunc_1_9 = 9;
  localparam th_myfunc_1_10 = 10;
  localparam th_myfunc_1_11 = 11;
  localparam th_myfunc_1_12 = 12;
  localparam th_myfunc_1_13 = 13;
  localparam th_myfunc_1_14 = 14;
  localparam th_myfunc_1_15 = 15;

  always @(posedge CLK) begin
    if(RST) begin
      th_myfunc_1 <= th_myfunc_1_init;
      _th_myfunc_1_called <= 0;
      _th_myfunc_1_tid_10 <= 0;
      _th_myfunc_1_size_11 <= 0;
      _th_myfunc_1_tid_12 <= 0;
      _th_myfunc_1_size_13 <= 0;
      _th_myfunc_1_i_14 <= 0;
      _tmp_3 <= 0;
      _th_myfunc_1_read_data_15 <= 0;
      _th_myfunc_1_write_data_16 <= 0;
    end else begin
      case(th_myfunc_1)
        th_myfunc_1_init: begin
          if(_th_myfunc_start[1] && (th_blink == 9)) begin
            _th_myfunc_1_called <= 1;
          end 
          if(_th_myfunc_start[1] && (th_blink == 9)) begin
            _th_myfunc_1_tid_10 <= _th_blink_tid_2;
          end 
          if(_th_myfunc_start[1] && (th_blink == 9)) begin
            _th_myfunc_1_size_11 <= _th_blink_size_0;
          end 
          if((th_blink == 9) && _th_myfunc_start[1]) begin
            th_myfunc_1 <= th_myfunc_1_1;
          end 
        end
        th_myfunc_1_1: begin
          _th_myfunc_1_tid_12 <= _th_myfunc_1_tid_10;
          _th_myfunc_1_size_13 <= _th_myfunc_1_size_11;
          th_myfunc_1 <= th_myfunc_1_2;
        end
        th_myfunc_1_2: begin
          if(!__myram_mutex_lock_reg || (__myram_mutex_lock_id == 1)) begin
            th_myfunc_1 <= th_myfunc_1_3;
          end 
        end
        th_myfunc_1_3: begin
          if(!(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 1))) begin
            th_myfunc_1 <= th_myfunc_1_2;
          end 
          if(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 1)) begin
            th_myfunc_1 <= th_myfunc_1_4;
          end 
        end
        th_myfunc_1_4: begin
          $display("Thread %d Lock", _th_myfunc_1_tid_12);
          th_myfunc_1 <= th_myfunc_1_5;
        end
        th_myfunc_1_5: begin
          _th_myfunc_1_i_14 <= 0;
          th_myfunc_1 <= th_myfunc_1_6;
        end
        th_myfunc_1_6: begin
          if(_th_myfunc_1_i_14 < _th_myfunc_1_size_13) begin
            th_myfunc_1 <= th_myfunc_1_7;
          end else begin
            th_myfunc_1 <= th_myfunc_1_13;
          end
        end
        th_myfunc_1_7: begin
          if(_tmp_2) begin
            _tmp_3 <= myram_0_rdata;
          end 
          if(_tmp_2) begin
            th_myfunc_1 <= th_myfunc_1_8;
          end 
        end
        th_myfunc_1_8: begin
          _th_myfunc_1_read_data_15 <= _tmp_3;
          th_myfunc_1 <= th_myfunc_1_9;
        end
        th_myfunc_1_9: begin
          _th_myfunc_1_write_data_16 <= _th_myfunc_1_read_data_15 + _th_myfunc_1_tid_12 + _th_myfunc_1_i_14;
          th_myfunc_1 <= th_myfunc_1_10;
        end
        th_myfunc_1_10: begin
          th_myfunc_1 <= th_myfunc_1_11;
        end
        th_myfunc_1_11: begin
          $display("Thread %d ram[%d] <- %d", _th_myfunc_1_tid_12, _th_myfunc_1_i_14, _th_myfunc_1_write_data_16);
          th_myfunc_1 <= th_myfunc_1_12;
        end
        th_myfunc_1_12: begin
          _th_myfunc_1_i_14 <= _th_myfunc_1_i_14 + 1;
          th_myfunc_1 <= th_myfunc_1_6;
        end
        th_myfunc_1_13: begin
          th_myfunc_1 <= th_myfunc_1_14;
        end
        th_myfunc_1_14: begin
          $display("Thread %d Unlock", _th_myfunc_1_tid_12);
          th_myfunc_1 <= th_myfunc_1_15;
        end
      endcase
    end
  end

  localparam th_myfunc_2_1 = 1;
  localparam th_myfunc_2_2 = 2;
  localparam th_myfunc_2_3 = 3;
  localparam th_myfunc_2_4 = 4;
  localparam th_myfunc_2_5 = 5;
  localparam th_myfunc_2_6 = 6;
  localparam th_myfunc_2_7 = 7;
  localparam th_myfunc_2_8 = 8;
  localparam th_myfunc_2_9 = 9;
  localparam th_myfunc_2_10 = 10;
  localparam th_myfunc_2_11 = 11;
  localparam th_myfunc_2_12 = 12;
  localparam th_myfunc_2_13 = 13;
  localparam th_myfunc_2_14 = 14;
  localparam th_myfunc_2_15 = 15;

  always @(posedge CLK) begin
    if(RST) begin
      th_myfunc_2 <= th_myfunc_2_init;
      _th_myfunc_2_called <= 0;
      _th_myfunc_2_tid_17 <= 0;
      _th_myfunc_2_size_18 <= 0;
      _th_myfunc_2_tid_19 <= 0;
      _th_myfunc_2_size_20 <= 0;
      _th_myfunc_2_i_21 <= 0;
      _tmp_5 <= 0;
      _th_myfunc_2_read_data_22 <= 0;
      _th_myfunc_2_write_data_23 <= 0;
    end else begin
      case(th_myfunc_2)
        th_myfunc_2_init: begin
          if(_th_myfunc_start[2] && (th_blink == 9)) begin
            _th_myfunc_2_called <= 1;
          end 
          if(_th_myfunc_start[2] && (th_blink == 9)) begin
            _th_myfunc_2_tid_17 <= _th_blink_tid_2;
          end 
          if(_th_myfunc_start[2] && (th_blink == 9)) begin
            _th_myfunc_2_size_18 <= _th_blink_size_0;
          end 
          if((th_blink == 9) && _th_myfunc_start[2]) begin
            th_myfunc_2 <= th_myfunc_2_1;
          end 
        end
        th_myfunc_2_1: begin
          _th_myfunc_2_tid_19 <= _th_myfunc_2_tid_17;
          _th_myfunc_2_size_20 <= _th_myfunc_2_size_18;
          th_myfunc_2 <= th_myfunc_2_2;
        end
        th_myfunc_2_2: begin
          if(!__myram_mutex_lock_reg || (__myram_mutex_lock_id == 2)) begin
            th_myfunc_2 <= th_myfunc_2_3;
          end 
        end
        th_myfunc_2_3: begin
          if(!(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 2))) begin
            th_myfunc_2 <= th_myfunc_2_2;
          end 
          if(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 2)) begin
            th_myfunc_2 <= th_myfunc_2_4;
          end 
        end
        th_myfunc_2_4: begin
          $display("Thread %d Lock", _th_myfunc_2_tid_19);
          th_myfunc_2 <= th_myfunc_2_5;
        end
        th_myfunc_2_5: begin
          _th_myfunc_2_i_21 <= 0;
          th_myfunc_2 <= th_myfunc_2_6;
        end
        th_myfunc_2_6: begin
          if(_th_myfunc_2_i_21 < _th_myfunc_2_size_20) begin
            th_myfunc_2 <= th_myfunc_2_7;
          end else begin
            th_myfunc_2 <= th_myfunc_2_13;
          end
        end
        th_myfunc_2_7: begin
          if(_tmp_4) begin
            _tmp_5 <= myram_0_rdata;
          end 
          if(_tmp_4) begin
            th_myfunc_2 <= th_myfunc_2_8;
          end 
        end
        th_myfunc_2_8: begin
          _th_myfunc_2_read_data_22 <= _tmp_5;
          th_myfunc_2 <= th_myfunc_2_9;
        end
        th_myfunc_2_9: begin
          _th_myfunc_2_write_data_23 <= _th_myfunc_2_read_data_22 + _th_myfunc_2_tid_19 + _th_myfunc_2_i_21;
          th_myfunc_2 <= th_myfunc_2_10;
        end
        th_myfunc_2_10: begin
          th_myfunc_2 <= th_myfunc_2_11;
        end
        th_myfunc_2_11: begin
          $display("Thread %d ram[%d] <- %d", _th_myfunc_2_tid_19, _th_myfunc_2_i_21, _th_myfunc_2_write_data_23);
          th_myfunc_2 <= th_myfunc_2_12;
        end
        th_myfunc_2_12: begin
          _th_myfunc_2_i_21 <= _th_myfunc_2_i_21 + 1;
          th_myfunc_2 <= th_myfunc_2_6;
        end
        th_myfunc_2_13: begin
          th_myfunc_2 <= th_myfunc_2_14;
        end
        th_myfunc_2_14: begin
          $display("Thread %d Unlock", _th_myfunc_2_tid_19);
          th_myfunc_2 <= th_myfunc_2_15;
        end
      endcase
    end
  end

  localparam th_myfunc_3_1 = 1;
  localparam th_myfunc_3_2 = 2;
  localparam th_myfunc_3_3 = 3;
  localparam th_myfunc_3_4 = 4;
  localparam th_myfunc_3_5 = 5;
  localparam th_myfunc_3_6 = 6;
  localparam th_myfunc_3_7 = 7;
  localparam th_myfunc_3_8 = 8;
  localparam th_myfunc_3_9 = 9;
  localparam th_myfunc_3_10 = 10;
  localparam th_myfunc_3_11 = 11;
  localparam th_myfunc_3_12 = 12;
  localparam th_myfunc_3_13 = 13;
  localparam th_myfunc_3_14 = 14;
  localparam th_myfunc_3_15 = 15;

  always @(posedge CLK) begin
    if(RST) begin
      th_myfunc_3 <= th_myfunc_3_init;
      _th_myfunc_3_called <= 0;
      _th_myfunc_3_tid_24 <= 0;
      _th_myfunc_3_size_25 <= 0;
      _th_myfunc_3_tid_26 <= 0;
      _th_myfunc_3_size_27 <= 0;
      _th_myfunc_3_i_28 <= 0;
      _tmp_7 <= 0;
      _th_myfunc_3_read_data_29 <= 0;
      _th_myfunc_3_write_data_30 <= 0;
    end else begin
      case(th_myfunc_3)
        th_myfunc_3_init: begin
          if(_th_myfunc_start[3] && (th_blink == 9)) begin
            _th_myfunc_3_called <= 1;
          end 
          if(_th_myfunc_start[3] && (th_blink == 9)) begin
            _th_myfunc_3_tid_24 <= _th_blink_tid_2;
          end 
          if(_th_myfunc_start[3] && (th_blink == 9)) begin
            _th_myfunc_3_size_25 <= _th_blink_size_0;
          end 
          if((th_blink == 9) && _th_myfunc_start[3]) begin
            th_myfunc_3 <= th_myfunc_3_1;
          end 
        end
        th_myfunc_3_1: begin
          _th_myfunc_3_tid_26 <= _th_myfunc_3_tid_24;
          _th_myfunc_3_size_27 <= _th_myfunc_3_size_25;
          th_myfunc_3 <= th_myfunc_3_2;
        end
        th_myfunc_3_2: begin
          if(!__myram_mutex_lock_reg || (__myram_mutex_lock_id == 3)) begin
            th_myfunc_3 <= th_myfunc_3_3;
          end 
        end
        th_myfunc_3_3: begin
          if(!(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 3))) begin
            th_myfunc_3 <= th_myfunc_3_2;
          end 
          if(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 3)) begin
            th_myfunc_3 <= th_myfunc_3_4;
          end 
        end
        th_myfunc_3_4: begin
          $display("Thread %d Lock", _th_myfunc_3_tid_26);
          th_myfunc_3 <= th_myfunc_3_5;
        end
        th_myfunc_3_5: begin
          _th_myfunc_3_i_28 <= 0;
          th_myfunc_3 <= th_myfunc_3_6;
        end
        th_myfunc_3_6: begin
          if(_th_myfunc_3_i_28 < _th_myfunc_3_size_27) begin
            th_myfunc_3 <= th_myfunc_3_7;
          end else begin
            th_myfunc_3 <= th_myfunc_3_13;
          end
        end
        th_myfunc_3_7: begin
          if(_tmp_6) begin
            _tmp_7 <= myram_0_rdata;
          end 
          if(_tmp_6) begin
            th_myfunc_3 <= th_myfunc_3_8;
          end 
        end
        th_myfunc_3_8: begin
          _th_myfunc_3_read_data_29 <= _tmp_7;
          th_myfunc_3 <= th_myfunc_3_9;
        end
        th_myfunc_3_9: begin
          _th_myfunc_3_write_data_30 <= _th_myfunc_3_read_data_29 + _th_myfunc_3_tid_26 + _th_myfunc_3_i_28;
          th_myfunc_3 <= th_myfunc_3_10;
        end
        th_myfunc_3_10: begin
          th_myfunc_3 <= th_myfunc_3_11;
        end
        th_myfunc_3_11: begin
          $display("Thread %d ram[%d] <- %d", _th_myfunc_3_tid_26, _th_myfunc_3_i_28, _th_myfunc_3_write_data_30);
          th_myfunc_3 <= th_myfunc_3_12;
        end
        th_myfunc_3_12: begin
          _th_myfunc_3_i_28 <= _th_myfunc_3_i_28 + 1;
          th_myfunc_3 <= th_myfunc_3_6;
        end
        th_myfunc_3_13: begin
          th_myfunc_3 <= th_myfunc_3_14;
        end
        th_myfunc_3_14: begin
          $display("Thread %d Unlock", _th_myfunc_3_tid_26);
          th_myfunc_3 <= th_myfunc_3_15;
        end
      endcase
    end
  end

  localparam th_myfunc_4_1 = 1;
  localparam th_myfunc_4_2 = 2;
  localparam th_myfunc_4_3 = 3;
  localparam th_myfunc_4_4 = 4;
  localparam th_myfunc_4_5 = 5;
  localparam th_myfunc_4_6 = 6;
  localparam th_myfunc_4_7 = 7;
  localparam th_myfunc_4_8 = 8;
  localparam th_myfunc_4_9 = 9;
  localparam th_myfunc_4_10 = 10;
  localparam th_myfunc_4_11 = 11;
  localparam th_myfunc_4_12 = 12;
  localparam th_myfunc_4_13 = 13;
  localparam th_myfunc_4_14 = 14;
  localparam th_myfunc_4_15 = 15;

  always @(posedge CLK) begin
    if(RST) begin
      th_myfunc_4 <= th_myfunc_4_init;
      _th_myfunc_4_called <= 0;
      _th_myfunc_4_tid_31 <= 0;
      _th_myfunc_4_size_32 <= 0;
      _th_myfunc_4_tid_33 <= 0;
      _th_myfunc_4_size_34 <= 0;
      _th_myfunc_4_i_35 <= 0;
      _tmp_9 <= 0;
      _th_myfunc_4_read_data_36 <= 0;
      _th_myfunc_4_write_data_37 <= 0;
    end else begin
      case(th_myfunc_4)
        th_myfunc_4_init: begin
          if(_th_myfunc_start[4] && (th_blink == 9)) begin
            _th_myfunc_4_called <= 1;
          end 
          if(_th_myfunc_start[4] && (th_blink == 9)) begin
            _th_myfunc_4_tid_31 <= _th_blink_tid_2;
          end 
          if(_th_myfunc_start[4] && (th_blink == 9)) begin
            _th_myfunc_4_size_32 <= _th_blink_size_0;
          end 
          if((th_blink == 9) && _th_myfunc_start[4]) begin
            th_myfunc_4 <= th_myfunc_4_1;
          end 
        end
        th_myfunc_4_1: begin
          _th_myfunc_4_tid_33 <= _th_myfunc_4_tid_31;
          _th_myfunc_4_size_34 <= _th_myfunc_4_size_32;
          th_myfunc_4 <= th_myfunc_4_2;
        end
        th_myfunc_4_2: begin
          if(!__myram_mutex_lock_reg || (__myram_mutex_lock_id == 4)) begin
            th_myfunc_4 <= th_myfunc_4_3;
          end 
        end
        th_myfunc_4_3: begin
          if(!(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 4))) begin
            th_myfunc_4 <= th_myfunc_4_2;
          end 
          if(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 4)) begin
            th_myfunc_4 <= th_myfunc_4_4;
          end 
        end
        th_myfunc_4_4: begin
          $display("Thread %d Lock", _th_myfunc_4_tid_33);
          th_myfunc_4 <= th_myfunc_4_5;
        end
        th_myfunc_4_5: begin
          _th_myfunc_4_i_35 <= 0;
          th_myfunc_4 <= th_myfunc_4_6;
        end
        th_myfunc_4_6: begin
          if(_th_myfunc_4_i_35 < _th_myfunc_4_size_34) begin
            th_myfunc_4 <= th_myfunc_4_7;
          end else begin
            th_myfunc_4 <= th_myfunc_4_13;
          end
        end
        th_myfunc_4_7: begin
          if(_tmp_8) begin
            _tmp_9 <= myram_0_rdata;
          end 
          if(_tmp_8) begin
            th_myfunc_4 <= th_myfunc_4_8;
          end 
        end
        th_myfunc_4_8: begin
          _th_myfunc_4_read_data_36 <= _tmp_9;
          th_myfunc_4 <= th_myfunc_4_9;
        end
        th_myfunc_4_9: begin
          _th_myfunc_4_write_data_37 <= _th_myfunc_4_read_data_36 + _th_myfunc_4_tid_33 + _th_myfunc_4_i_35;
          th_myfunc_4 <= th_myfunc_4_10;
        end
        th_myfunc_4_10: begin
          th_myfunc_4 <= th_myfunc_4_11;
        end
        th_myfunc_4_11: begin
          $display("Thread %d ram[%d] <- %d", _th_myfunc_4_tid_33, _th_myfunc_4_i_35, _th_myfunc_4_write_data_37);
          th_myfunc_4 <= th_myfunc_4_12;
        end
        th_myfunc_4_12: begin
          _th_myfunc_4_i_35 <= _th_myfunc_4_i_35 + 1;
          th_myfunc_4 <= th_myfunc_4_6;
        end
        th_myfunc_4_13: begin
          th_myfunc_4 <= th_myfunc_4_14;
        end
        th_myfunc_4_14: begin
          $display("Thread %d Unlock", _th_myfunc_4_tid_33);
          th_myfunc_4 <= th_myfunc_4_15;
        end
      endcase
    end
  end

  localparam th_myfunc_5_1 = 1;
  localparam th_myfunc_5_2 = 2;
  localparam th_myfunc_5_3 = 3;
  localparam th_myfunc_5_4 = 4;
  localparam th_myfunc_5_5 = 5;
  localparam th_myfunc_5_6 = 6;
  localparam th_myfunc_5_7 = 7;
  localparam th_myfunc_5_8 = 8;
  localparam th_myfunc_5_9 = 9;
  localparam th_myfunc_5_10 = 10;
  localparam th_myfunc_5_11 = 11;
  localparam th_myfunc_5_12 = 12;
  localparam th_myfunc_5_13 = 13;
  localparam th_myfunc_5_14 = 14;
  localparam th_myfunc_5_15 = 15;

  always @(posedge CLK) begin
    if(RST) begin
      th_myfunc_5 <= th_myfunc_5_init;
      _th_myfunc_5_called <= 0;
      _th_myfunc_5_tid_38 <= 0;
      _th_myfunc_5_size_39 <= 0;
      _th_myfunc_5_tid_40 <= 0;
      _th_myfunc_5_size_41 <= 0;
      _th_myfunc_5_i_42 <= 0;
      _tmp_11 <= 0;
      _th_myfunc_5_read_data_43 <= 0;
      _th_myfunc_5_write_data_44 <= 0;
    end else begin
      case(th_myfunc_5)
        th_myfunc_5_init: begin
          if(_th_myfunc_start[5] && (th_blink == 9)) begin
            _th_myfunc_5_called <= 1;
          end 
          if(_th_myfunc_start[5] && (th_blink == 9)) begin
            _th_myfunc_5_tid_38 <= _th_blink_tid_2;
          end 
          if(_th_myfunc_start[5] && (th_blink == 9)) begin
            _th_myfunc_5_size_39 <= _th_blink_size_0;
          end 
          if((th_blink == 9) && _th_myfunc_start[5]) begin
            th_myfunc_5 <= th_myfunc_5_1;
          end 
        end
        th_myfunc_5_1: begin
          _th_myfunc_5_tid_40 <= _th_myfunc_5_tid_38;
          _th_myfunc_5_size_41 <= _th_myfunc_5_size_39;
          th_myfunc_5 <= th_myfunc_5_2;
        end
        th_myfunc_5_2: begin
          if(!__myram_mutex_lock_reg || (__myram_mutex_lock_id == 5)) begin
            th_myfunc_5 <= th_myfunc_5_3;
          end 
        end
        th_myfunc_5_3: begin
          if(!(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 5))) begin
            th_myfunc_5 <= th_myfunc_5_2;
          end 
          if(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 5)) begin
            th_myfunc_5 <= th_myfunc_5_4;
          end 
        end
        th_myfunc_5_4: begin
          $display("Thread %d Lock", _th_myfunc_5_tid_40);
          th_myfunc_5 <= th_myfunc_5_5;
        end
        th_myfunc_5_5: begin
          _th_myfunc_5_i_42 <= 0;
          th_myfunc_5 <= th_myfunc_5_6;
        end
        th_myfunc_5_6: begin
          if(_th_myfunc_5_i_42 < _th_myfunc_5_size_41) begin
            th_myfunc_5 <= th_myfunc_5_7;
          end else begin
            th_myfunc_5 <= th_myfunc_5_13;
          end
        end
        th_myfunc_5_7: begin
          if(_tmp_10) begin
            _tmp_11 <= myram_0_rdata;
          end 
          if(_tmp_10) begin
            th_myfunc_5 <= th_myfunc_5_8;
          end 
        end
        th_myfunc_5_8: begin
          _th_myfunc_5_read_data_43 <= _tmp_11;
          th_myfunc_5 <= th_myfunc_5_9;
        end
        th_myfunc_5_9: begin
          _th_myfunc_5_write_data_44 <= _th_myfunc_5_read_data_43 + _th_myfunc_5_tid_40 + _th_myfunc_5_i_42;
          th_myfunc_5 <= th_myfunc_5_10;
        end
        th_myfunc_5_10: begin
          th_myfunc_5 <= th_myfunc_5_11;
        end
        th_myfunc_5_11: begin
          $display("Thread %d ram[%d] <- %d", _th_myfunc_5_tid_40, _th_myfunc_5_i_42, _th_myfunc_5_write_data_44);
          th_myfunc_5 <= th_myfunc_5_12;
        end
        th_myfunc_5_12: begin
          _th_myfunc_5_i_42 <= _th_myfunc_5_i_42 + 1;
          th_myfunc_5 <= th_myfunc_5_6;
        end
        th_myfunc_5_13: begin
          th_myfunc_5 <= th_myfunc_5_14;
        end
        th_myfunc_5_14: begin
          $display("Thread %d Unlock", _th_myfunc_5_tid_40);
          th_myfunc_5 <= th_myfunc_5_15;
        end
      endcase
    end
  end

  localparam th_myfunc_6_1 = 1;
  localparam th_myfunc_6_2 = 2;
  localparam th_myfunc_6_3 = 3;
  localparam th_myfunc_6_4 = 4;
  localparam th_myfunc_6_5 = 5;
  localparam th_myfunc_6_6 = 6;
  localparam th_myfunc_6_7 = 7;
  localparam th_myfunc_6_8 = 8;
  localparam th_myfunc_6_9 = 9;
  localparam th_myfunc_6_10 = 10;
  localparam th_myfunc_6_11 = 11;
  localparam th_myfunc_6_12 = 12;
  localparam th_myfunc_6_13 = 13;
  localparam th_myfunc_6_14 = 14;
  localparam th_myfunc_6_15 = 15;

  always @(posedge CLK) begin
    if(RST) begin
      th_myfunc_6 <= th_myfunc_6_init;
      _th_myfunc_6_called <= 0;
      _th_myfunc_6_tid_45 <= 0;
      _th_myfunc_6_size_46 <= 0;
      _th_myfunc_6_tid_47 <= 0;
      _th_myfunc_6_size_48 <= 0;
      _th_myfunc_6_i_49 <= 0;
      _tmp_13 <= 0;
      _th_myfunc_6_read_data_50 <= 0;
      _th_myfunc_6_write_data_51 <= 0;
    end else begin
      case(th_myfunc_6)
        th_myfunc_6_init: begin
          if(_th_myfunc_start[6] && (th_blink == 9)) begin
            _th_myfunc_6_called <= 1;
          end 
          if(_th_myfunc_start[6] && (th_blink == 9)) begin
            _th_myfunc_6_tid_45 <= _th_blink_tid_2;
          end 
          if(_th_myfunc_start[6] && (th_blink == 9)) begin
            _th_myfunc_6_size_46 <= _th_blink_size_0;
          end 
          if((th_blink == 9) && _th_myfunc_start[6]) begin
            th_myfunc_6 <= th_myfunc_6_1;
          end 
        end
        th_myfunc_6_1: begin
          _th_myfunc_6_tid_47 <= _th_myfunc_6_tid_45;
          _th_myfunc_6_size_48 <= _th_myfunc_6_size_46;
          th_myfunc_6 <= th_myfunc_6_2;
        end
        th_myfunc_6_2: begin
          if(!__myram_mutex_lock_reg || (__myram_mutex_lock_id == 6)) begin
            th_myfunc_6 <= th_myfunc_6_3;
          end 
        end
        th_myfunc_6_3: begin
          if(!(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 6))) begin
            th_myfunc_6 <= th_myfunc_6_2;
          end 
          if(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 6)) begin
            th_myfunc_6 <= th_myfunc_6_4;
          end 
        end
        th_myfunc_6_4: begin
          $display("Thread %d Lock", _th_myfunc_6_tid_47);
          th_myfunc_6 <= th_myfunc_6_5;
        end
        th_myfunc_6_5: begin
          _th_myfunc_6_i_49 <= 0;
          th_myfunc_6 <= th_myfunc_6_6;
        end
        th_myfunc_6_6: begin
          if(_th_myfunc_6_i_49 < _th_myfunc_6_size_48) begin
            th_myfunc_6 <= th_myfunc_6_7;
          end else begin
            th_myfunc_6 <= th_myfunc_6_13;
          end
        end
        th_myfunc_6_7: begin
          if(_tmp_12) begin
            _tmp_13 <= myram_0_rdata;
          end 
          if(_tmp_12) begin
            th_myfunc_6 <= th_myfunc_6_8;
          end 
        end
        th_myfunc_6_8: begin
          _th_myfunc_6_read_data_50 <= _tmp_13;
          th_myfunc_6 <= th_myfunc_6_9;
        end
        th_myfunc_6_9: begin
          _th_myfunc_6_write_data_51 <= _th_myfunc_6_read_data_50 + _th_myfunc_6_tid_47 + _th_myfunc_6_i_49;
          th_myfunc_6 <= th_myfunc_6_10;
        end
        th_myfunc_6_10: begin
          th_myfunc_6 <= th_myfunc_6_11;
        end
        th_myfunc_6_11: begin
          $display("Thread %d ram[%d] <- %d", _th_myfunc_6_tid_47, _th_myfunc_6_i_49, _th_myfunc_6_write_data_51);
          th_myfunc_6 <= th_myfunc_6_12;
        end
        th_myfunc_6_12: begin
          _th_myfunc_6_i_49 <= _th_myfunc_6_i_49 + 1;
          th_myfunc_6 <= th_myfunc_6_6;
        end
        th_myfunc_6_13: begin
          th_myfunc_6 <= th_myfunc_6_14;
        end
        th_myfunc_6_14: begin
          $display("Thread %d Unlock", _th_myfunc_6_tid_47);
          th_myfunc_6 <= th_myfunc_6_15;
        end
      endcase
    end
  end

  localparam th_myfunc_7_1 = 1;
  localparam th_myfunc_7_2 = 2;
  localparam th_myfunc_7_3 = 3;
  localparam th_myfunc_7_4 = 4;
  localparam th_myfunc_7_5 = 5;
  localparam th_myfunc_7_6 = 6;
  localparam th_myfunc_7_7 = 7;
  localparam th_myfunc_7_8 = 8;
  localparam th_myfunc_7_9 = 9;
  localparam th_myfunc_7_10 = 10;
  localparam th_myfunc_7_11 = 11;
  localparam th_myfunc_7_12 = 12;
  localparam th_myfunc_7_13 = 13;
  localparam th_myfunc_7_14 = 14;
  localparam th_myfunc_7_15 = 15;

  always @(posedge CLK) begin
    if(RST) begin
      th_myfunc_7 <= th_myfunc_7_init;
      _th_myfunc_7_called <= 0;
      _th_myfunc_7_tid_52 <= 0;
      _th_myfunc_7_size_53 <= 0;
      _th_myfunc_7_tid_54 <= 0;
      _th_myfunc_7_size_55 <= 0;
      _th_myfunc_7_i_56 <= 0;
      _tmp_15 <= 0;
      _th_myfunc_7_read_data_57 <= 0;
      _th_myfunc_7_write_data_58 <= 0;
    end else begin
      case(th_myfunc_7)
        th_myfunc_7_init: begin
          if(_th_myfunc_start[7] && (th_blink == 9)) begin
            _th_myfunc_7_called <= 1;
          end 
          if(_th_myfunc_start[7] && (th_blink == 9)) begin
            _th_myfunc_7_tid_52 <= _th_blink_tid_2;
          end 
          if(_th_myfunc_start[7] && (th_blink == 9)) begin
            _th_myfunc_7_size_53 <= _th_blink_size_0;
          end 
          if((th_blink == 9) && _th_myfunc_start[7]) begin
            th_myfunc_7 <= th_myfunc_7_1;
          end 
        end
        th_myfunc_7_1: begin
          _th_myfunc_7_tid_54 <= _th_myfunc_7_tid_52;
          _th_myfunc_7_size_55 <= _th_myfunc_7_size_53;
          th_myfunc_7 <= th_myfunc_7_2;
        end
        th_myfunc_7_2: begin
          if(!__myram_mutex_lock_reg || (__myram_mutex_lock_id == 7)) begin
            th_myfunc_7 <= th_myfunc_7_3;
          end 
        end
        th_myfunc_7_3: begin
          if(!(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 7))) begin
            th_myfunc_7 <= th_myfunc_7_2;
          end 
          if(__myram_mutex_lock_reg && (__myram_mutex_lock_id == 7)) begin
            th_myfunc_7 <= th_myfunc_7_4;
          end 
        end
        th_myfunc_7_4: begin
          $display("Thread %d Lock", _th_myfunc_7_tid_54);
          th_myfunc_7 <= th_myfunc_7_5;
        end
        th_myfunc_7_5: begin
          _th_myfunc_7_i_56 <= 0;
          th_myfunc_7 <= th_myfunc_7_6;
        end
        th_myfunc_7_6: begin
          if(_th_myfunc_7_i_56 < _th_myfunc_7_size_55) begin
            th_myfunc_7 <= th_myfunc_7_7;
          end else begin
            th_myfunc_7 <= th_myfunc_7_13;
          end
        end
        th_myfunc_7_7: begin
          if(_tmp_14) begin
            _tmp_15 <= myram_0_rdata;
          end 
          if(_tmp_14) begin
            th_myfunc_7 <= th_myfunc_7_8;
          end 
        end
        th_myfunc_7_8: begin
          _th_myfunc_7_read_data_57 <= _tmp_15;
          th_myfunc_7 <= th_myfunc_7_9;
        end
        th_myfunc_7_9: begin
          _th_myfunc_7_write_data_58 <= _th_myfunc_7_read_data_57 + _th_myfunc_7_tid_54 + _th_myfunc_7_i_56;
          th_myfunc_7 <= th_myfunc_7_10;
        end
        th_myfunc_7_10: begin
          th_myfunc_7 <= th_myfunc_7_11;
        end
        th_myfunc_7_11: begin
          $display("Thread %d ram[%d] <- %d", _th_myfunc_7_tid_54, _th_myfunc_7_i_56, _th_myfunc_7_write_data_58);
          th_myfunc_7 <= th_myfunc_7_12;
        end
        th_myfunc_7_12: begin
          _th_myfunc_7_i_56 <= _th_myfunc_7_i_56 + 1;
          th_myfunc_7 <= th_myfunc_7_6;
        end
        th_myfunc_7_13: begin
          th_myfunc_7 <= th_myfunc_7_14;
        end
        th_myfunc_7_14: begin
          $display("Thread %d Unlock", _th_myfunc_7_tid_54);
          th_myfunc_7 <= th_myfunc_7_15;
        end
      endcase
    end
  end


  always @(posedge CLK) begin
    if(RST) begin
      __myram_mutex_lock_reg <= 0;
      __myram_mutex_lock_id <= 0;
    end else begin
      if((th_myfunc_0 == 2) && !__myram_mutex_lock_reg) begin
        __myram_mutex_lock_reg <= 1;
        __myram_mutex_lock_id <= 0;
      end 
      if((th_myfunc_0 == 13) && (__myram_mutex_lock_id == 0)) begin
        __myram_mutex_lock_reg <= 0;
      end 
      if((th_myfunc_1 == 2) && !__myram_mutex_lock_reg) begin
        __myram_mutex_lock_reg <= 1;
        __myram_mutex_lock_id <= 1;
      end 
      if((th_myfunc_1 == 13) && (__myram_mutex_lock_id == 1)) begin
        __myram_mutex_lock_reg <= 0;
      end 
      if((th_myfunc_2 == 2) && !__myram_mutex_lock_reg) begin
        __myram_mutex_lock_reg <= 1;
        __myram_mutex_lock_id <= 2;
      end 
      if((th_myfunc_2 == 13) && (__myram_mutex_lock_id == 2)) begin
        __myram_mutex_lock_reg <= 0;
      end 
      if((th_myfunc_3 == 2) && !__myram_mutex_lock_reg) begin
        __myram_mutex_lock_reg <= 1;
        __myram_mutex_lock_id <= 3;
      end 
      if((th_myfunc_3 == 13) && (__myram_mutex_lock_id == 3)) begin
        __myram_mutex_lock_reg <= 0;
      end 
      if((th_myfunc_4 == 2) && !__myram_mutex_lock_reg) begin
        __myram_mutex_lock_reg <= 1;
        __myram_mutex_lock_id <= 4;
      end 
      if((th_myfunc_4 == 13) && (__myram_mutex_lock_id == 4)) begin
        __myram_mutex_lock_reg <= 0;
      end 
      if((th_myfunc_5 == 2) && !__myram_mutex_lock_reg) begin
        __myram_mutex_lock_reg <= 1;
        __myram_mutex_lock_id <= 5;
      end 
      if((th_myfunc_5 == 13) && (__myram_mutex_lock_id == 5)) begin
        __myram_mutex_lock_reg <= 0;
      end 
      if((th_myfunc_6 == 2) && !__myram_mutex_lock_reg) begin
        __myram_mutex_lock_reg <= 1;
        __myram_mutex_lock_id <= 6;
      end 
      if((th_myfunc_6 == 13) && (__myram_mutex_lock_id == 6)) begin
        __myram_mutex_lock_reg <= 0;
      end 
      if((th_myfunc_7 == 2) && !__myram_mutex_lock_reg) begin
        __myram_mutex_lock_reg <= 1;
        __myram_mutex_lock_id <= 7;
      end 
      if((th_myfunc_7 == 13) && (__myram_mutex_lock_id == 7)) begin
        __myram_mutex_lock_reg <= 0;
      end 
    end
  end


endmodule



module myram
(
  input CLK,
  input [10-1:0] myram_0_addr,
  output [32-1:0] myram_0_rdata,
  input [32-1:0] myram_0_wdata,
  input myram_0_wenable
);

  reg [10-1:0] myram_0_daddr;
  reg [32-1:0] mem [0:1024-1];

  always @(posedge CLK) begin
    if(myram_0_wenable) begin
      mem[myram_0_addr] <= myram_0_wdata;
    end 
    myram_0_daddr <= myram_0_addr;
  end

  assign myram_0_rdata = mem[myram_0_daddr];

endmodule
"""


def test():
    veriloggen.reset()
    test_module = thread_ram_own_mutex.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.abspath(__file__)))))

from veriloggen import *
import veriloggen.thread as vthread
import veriloggen.types.axi as axi
import veriloggen.types.ipcore as ipcore


def mkLed():
    m = Module('blinkled')
    clk = m.Input('CLK')
    rst = m.Input('RST')
    led = m.OutputReg('led', 8, initval=0)

    datawidth = 32
    addrwidth = 10

    saxi = vthread.AXISLiteRegister(m, 'saxi', clk, rst, datawidth)

    def wait(fsm, sleep):
        cnt = fsm.m.TmpReg(32, initval=0)
        fsm.If(cnt < sleep)(
            cnt.inc()
        )
        fsm.If(cnt >= sleep)(
            cnt(0)
        )
        fsm.Then().goto_next()

    def blink(size):
        while True:
            # wait start
            saxi.wait_flag(0, value=1, resetvalue=0)
            # reset done
            saxi.write(3, 0)

            sleep = saxi.read(1)
            size = saxi.read(2)

            for i in range(size):
                wait(sleep)
                led.value += 1

            # done
            saxi.write_flag(3, 1, resetvalue=0)

    th = vthread.Thread(m, 'th_blink', clk, rst, blink)
    th.add_intrinsics(wait)
    fsm = th.start(16)

    return m


def mkTest():
    m = Module('test')

    # target instance
    led = mkLed()

    # copy paras and ports
    params = m.copy_params(led)
    ports = m.copy_sim_ports(led)

    clk = ports['CLK']
    rst = ports['RST']

    # AXI-Slave controller
    _saxi = vthread.AXIMLite(m, '_saxi', clk, rst, noio=True)
    _saxi.connect(ports, 'saxi')

    # Timer
    counter = m.Reg('counter', 32, initval=0)
    seq = Seq(m, 'seq', clk, rst)
    seq(
        counter.inc()
    )

    def ctrl():
        for i in range(100):
            pass

        awaddr = 4
        sleep = 16
        print('# sleep = %d' % sleep)
        _saxi.write(awaddr, sleep)

        awaddr = 8
        size = 128
        print('# size = %d' % size)
        _saxi.write(awaddr, size)

        awaddr = 0
        start_time = counter
        print('# start time = %d' % start_time)
        _saxi.write(awaddr, 1)

        araddr = 12
        v = _saxi.read(araddr)
        while v == 0:
            v = _saxi.read(araddr)

        end_time = counter
        print('# end time = %d' % end_time)
        time = end_time - start_time
        print('# exec time = %d' % time)

    th = vthread.Thread(m, 'th_ctrl', clk, rst, ctrl)
    fsm = th.start()

    uut = m.Instance(led, 'uut',
                     params=m.connect_params(led),
                     ports=m.connect_ports(led))

    simulation.setup_waveform(m, uut)
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    init.add(
        Delay(100000),
        Systask('finish'),
    )

    return m

if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)

    simcode = """

reg [31:0] counter;
always @(posedge sim_clk) begin
  if(!sim_resetn) begin
    counter <= 0;
  end else begin
    counter <= counter + 1;
  end
end

reg [31:0] _start_time;
reg [31:0] _end_time;
reg [31:0] _time;

reg [31:0] _addr;
reg [31:0] _data;
initial begin
  #1000;
  _addr = 4;
  _data = 16;
  $display("# sleep = %d", _data);
  slave_write_ipgen_slave_lite_memory_saxi_0(_data, _addr);

  _addr = 8;
  _data = 128;
  $display("# size = %d", _data);
  slave_write_ipgen_slave_lite_memory_saxi_0(_data, _addr);

  _addr = 0;
  _data = 1;
  _start_time = counter;
  $display("# start time = %d", _start_time);
  slave_write_ipgen_slave_lite_memory_saxi_0(_data, _addr);

  _addr = 12;
  _data = 0;
  while(_data == 0) begin
    slave_read_ipgen_slave_lite_memory_saxi_0(_data, _addr);
    nclk();
  end
  _end_time = counter;
  $display("# end time = %d", _end_time);
  _time = _end_time - _start_time;
  $display("# exec time = %d", _time);

  #10000;
  $finish;
end
"""

    m = mkLed()
    ipcore.to_ipcore(m, simcode=simcode, iftype='axi')



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import seq_multiple_add

expected_verilog = """
module test;
  reg CLK;
  reg RST;
  reg [32-1:0] idata_00;
  reg [32-1:0] idata_01;
  reg [32-1:0] idata_02;
  reg [32-1:0] idata_03;
  reg [32-1:0] idata_04;
  reg [32-1:0] idata_05;
  reg [32-1:0] idata_06;
  reg [32-1:0] idata_07;
  reg ivalid_00;
  reg ivalid_01;
  reg ivalid_02;
  reg ivalid_03;
  reg ivalid_04;
  reg ivalid_05;
  reg ivalid_06;
  reg ivalid_07;
  wire [32-1:0] odata;
  wire ovalid;

  blinkled
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .idata_00(idata_00),
    .idata_01(idata_01),
    .idata_02(idata_02),
    .idata_03(idata_03),
    .idata_04(idata_04),
    .idata_05(idata_05),
    .idata_06(idata_06),
    .idata_07(idata_07),
    .ivalid_00(ivalid_00),
    .ivalid_01(ivalid_01),
    .ivalid_02(ivalid_02),
    .ivalid_03(ivalid_03),
    .ivalid_04(ivalid_04),
    .ivalid_05(ivalid_05),
    .ivalid_06(ivalid_06),
    .ivalid_07(ivalid_07),
    .odata(odata),
    .ovalid(ovalid)
  );

  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end

  initial begin
    CLK = 0;
    forever begin
      #5 CLK = (!CLK);
    end
  end

  initial begin
    RST = 0;
    idata_00 = 0;
    idata_01 = 0;
    idata_02 = 0;
    idata_03 = 0;
    idata_04 = 0;
    idata_05 = 0;
    idata_06 = 0;
    idata_07 = 0;
    ivalid_00 = 0;
    ivalid_01 = 0;
    ivalid_02 = 0;
    ivalid_03 = 0;
    ivalid_04 = 0;
    ivalid_05 = 0;
    ivalid_06 = 0;
    ivalid_07 = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #1000;
    @(posedge CLK);
    #1;
    idata_00 = 0;
    idata_01 = 0;
    idata_02 = 0;
    idata_03 = 0;
    idata_04 = 0;
    idata_05 = 0;
    idata_06 = 0;
    idata_07 = 0;
    ivalid_00 = 0;
    ivalid_01 = 0;
    ivalid_02 = 0;
    ivalid_03 = 0;
    ivalid_04 = 0;
    ivalid_05 = 0;
    ivalid_06 = 0;
    ivalid_07 = 0;
    @(posedge CLK);
    #1;
    idata_00 = 1;
    idata_01 = 2;
    idata_02 = 3;
    idata_03 = 4;
    idata_04 = 5;
    idata_05 = 6;
    idata_06 = 7;
    idata_07 = 8;
    ivalid_00 = 1;
    ivalid_01 = 1;
    ivalid_02 = 1;
    ivalid_03 = 1;
    ivalid_04 = 1;
    ivalid_05 = 1;
    ivalid_06 = 1;
    ivalid_07 = 1;
    @(posedge CLK);
    #1;
    idata_00 = 1;
    idata_01 = 1;
    idata_02 = 1;
    idata_03 = 1;
    idata_04 = 1;
    idata_05 = 1;
    idata_06 = 1;
    idata_07 = 1;
    ivalid_00 = 0;
    ivalid_01 = 0;
    ivalid_02 = 0;
    ivalid_03 = 0;
    ivalid_04 = 0;
    ivalid_05 = 0;
    ivalid_06 = 0;
    ivalid_07 = 0;
    @(posedge CLK);
    #1;
    idata_00 = 10;
    idata_01 = 11;
    idata_02 = 12;
    idata_03 = 13;
    idata_04 = 14;
    idata_05 = 15;
    idata_06 = 16;
    idata_07 = 17;
    ivalid_00 = 1;
    ivalid_01 = 1;
    ivalid_02 = 1;
    ivalid_03 = 1;
    ivalid_04 = 1;
    ivalid_05 = 1;
    ivalid_06 = 1;
    ivalid_07 = 1;
    @(posedge CLK);
    #1;
    ivalid_00 = 0;
    ivalid_01 = 0;
    ivalid_02 = 0;
    ivalid_03 = 0;
    ivalid_04 = 0;
    ivalid_05 = 0;
    ivalid_06 = 0;
    ivalid_07 = 0;
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    $finish;
  end

endmodule

module blinkled
  (
   input CLK,
   input RST,
   input [32-1:0] idata_00,
   input [32-1:0] idata_01,
   input [32-1:0] idata_02,
   input [32-1:0] idata_03,
   input [32-1:0] idata_04,
   input [32-1:0] idata_05,
   input [32-1:0] idata_06,
   input [32-1:0] idata_07,
   input ivalid_00,
   input ivalid_01,
   input ivalid_02,
   input ivalid_03,
   input ivalid_04,
   input ivalid_05,
   input ivalid_06,
   input ivalid_07,
   output reg [32-1:0] odata,
   output reg ovalid
  );

  reg [32-1:0] _tmp_0;
  reg _tmp_1;
  reg [32-1:0] _tmp_2;
  reg _tmp_3;
  reg [32-1:0] _tmp_4;
  reg _tmp_5;
  reg [32-1:0] _tmp_6;
  reg _tmp_7;
  reg [32-1:0] _tmp_8;
  reg _tmp_9;
  reg [32-1:0] _tmp_10;
  reg _tmp_11;
  reg [32-1:0] _tmp_12;
  reg _tmp_13;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_0 <= 0;
      _tmp_1 <= 0;
      _tmp_2 <= 0;
      _tmp_3 <= 0;
      _tmp_4 <= 0;
      _tmp_5 <= 0;
      _tmp_6 <= 0;
      _tmp_7 <= 0;
      _tmp_8 <= 0;
      _tmp_9 <= 0;
      _tmp_10 <= 0;
      _tmp_11 <= 0;
      _tmp_12 <= 0;
      _tmp_13 <= 0;
      odata <= 0;
      ovalid <= 0;
    end else begin
      if((ivalid_00 && ivalid_01)) begin
        _tmp_0 <= (idata_00 + idata_01);
      end 
      _tmp_1 <= (ivalid_00 && ivalid_01);
      if((ivalid_02 && ivalid_03)) begin
        _tmp_2 <= (idata_02 + idata_03);
      end 
      _tmp_3 <= (ivalid_02 && ivalid_03);
      if((ivalid_04 && ivalid_05)) begin
        _tmp_4 <= (idata_04 + idata_05);
      end 
      _tmp_5 <= (ivalid_04 && ivalid_05);
      if((ivalid_06 && ivalid_07)) begin
        _tmp_6 <= (idata_06 + idata_07);
      end 
      _tmp_7 <= (ivalid_06 && ivalid_07);
      if((_tmp_1 && _tmp_3)) begin
        _tmp_8 <= (_tmp_0 + _tmp_2);
      end 
      _tmp_9 <= (_tmp_1 && _tmp_3);
      if((_tmp_5 && _tmp_7)) begin
        _tmp_10 <= (_tmp_4 + _tmp_6);
      end 
      _tmp_11 <= (_tmp_5 && _tmp_7);
      if((_tmp_9 && _tmp_11)) begin
        _tmp_12 <= (_tmp_8 + _tmp_10);
      end 
      _tmp_13 <= (_tmp_9 && _tmp_11);
      odata <= _tmp_12;
      ovalid <= _tmp_13;
    end
  end

endmodule
"""

def test():
    veriloggen.reset()
    test_module = seq_multiple_add.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))

from veriloggen import *

def mkLed():
    m = Module('blinkled')
    width = m.Parameter('WIDTH', 8)
    clk = m.Input('CLK')
    rst = m.Input('RST')
    led = m.OutputReg('LED', width)
    count = m.Reg('count', 32)

    m.Always(Posedge(clk))(
        If(rst)(
            count(0)
        ).Else(
            If(count == 1023)(
                count(0)
            ).Else(
                count(count + 1)
            )
        ))
    
    m.Always(Posedge(clk))(
        If(rst)(
            # Create a repeat object directly
            led( Repeat(Int(0, width=1, base=2), width) )
        ).Else(
            If(count == 1024 - 1)(
                # Create a repeat object by calling repeat() method
                led( led + Int(1, width=1, base=2).repeat(width) )
            )
        ))

    return m

if __name__ == '__main__':
    led = mkLed()
    verilog = led.to_verilog()
    print(verilog)
